function zhChart(options) {
	var defaultOptions = {
			width: "100%",
			height: "100%",
			fontName: "'Droid Sans', Helvetica, Arial, sans-serif",
			padding: {
				left: 20,
				top: 10,
				right: 20,
				bottom: 30,
				title: 15
			},
			dataSource: {
				data: []
			},
			showTooltip: true,
			showValueLabels: true,
			showDrillUpButton: true,
			xAxis: {
				position: "bottom",
				type: "points",
				bandDimensions: {
					type: "auto",
					gap: 0.1
				},
				ticks: {
					maxWidth: undefined,
					maxLines: 2,
				}
			},
			legend: {
				position: "top right",
				margin: 10,
				visible: true,
				closed: false
			},
			invertedAxes: false,
			scrollable: true,
			events: {}
		};
	var chart, chartElement;
	var chartContainer, svgContainer;
	var svgContainerWidth, svgContainerHeight;
	var titlePadding = 10;
	var _duration = 1000;
	var drillPath = [];
	var axisDimensions = {
		left: 0,
		top: 0,
		right: 0,
		bottom: 0
	};
	var chartPanning = false;

	/* UTIL */
	function isUndefined(value) {
		return typeof(value) === "undefined";
	}

	function isUndefinedOrNull(value) {
		return isUndefined(value) || isNull(value);
	}
	
	function isNull(value) {
		return value === null;
	}

	function isFunction(value) {
		return Object.prototype.toString.call(value) === '[object Function]';
	}

	function isMobile() {
		return ( /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) );
	}

	function formatValue(value, format) {
		/* jslint evil: true */
		return (new Function("value", "return "+format))(value);
	}

	function colorExpression(value, defaultColor, expression) {
		/* jslint evil: true */
		return (new Function("value, defaultColor", "return "+expression))(value, defaultColor);
	}

	function buildDebounceMethod(method, milisecondsToDelay, firstCall) {
		var timeout;
		firstCall = typeof firstCall != "boolean" ? false : firstCall;
		return function() {
			var obj = this,
				args = arguments;

			function delayed() {
				if (!firstCall) {
					method.apply(obj, args);
				}
				timeout = null;
			}
			if (timeout) {
				clearTimeout(timeout);
			} else if (firstCall) {
				method.apply(obj, args);
			}
			timeout = setTimeout(delayed, milisecondsToDelay || 100);
		};
	}

	function initChart(element) {
		chart = jQuery.extend(true, {}, defaultOptions, options || {});
		chart.id = Math.round(Math.random() * 1000000);
		chartElement = element || chartElement;

		chartContainer = d3.select(chartElement || "body");
		chartContainer.classed("zh-chart-container", true);
		chartContainer.node()._chart = _chart;
		d3.select(chartContainer.node().parentNode).style("overflow", "hidden");

		svgContainer = chartContainer.select("svg");
		if (!svgContainer.empty()) {
			svgContainer.remove();
		}

		svgContainer = chartContainer.append("svg")
			.attr("id", chart.id)
			.style("font-family", chart.fontName);

		chart.clippingPath = svgContainer.append("defs")
			.append("clipPath")
			.attr("id", "clip-path-"+chart.id)
			.append("rect")
			.attr("fill", "none");

		svgContainer = svgContainer.append("g");

		if (chart.invertedAxes) { // add x-grid container
			svgContainer.append("g").attr("class","x-grid");
		}

		if (chart.scrollable) {
			svgContainer.append("rect")
				.attr("class","panning-controller")
				.attr("fill","transparent");
		}

		initTooltip();
	}

	function resizeChart() {
		var chartContainerWidth, chartContainerHeight;

		chartContainerWidth = chartContainer.node().clientWidth - parseInt(window.getComputedStyle(chartContainer.node(), null).getPropertyValue("padding-left")) - parseInt(window.getComputedStyle(chartContainer.node(), null).getPropertyValue("padding-right"));
		chartContainerHeight = chartContainer.node().clientHeight - parseInt(window.getComputedStyle(chartContainer.node(), null).getPropertyValue("padding-top")) - parseInt(window.getComputedStyle(chartContainer.node(), null).getPropertyValue("padding-bottom"));
		if (chart.width.indexOf('%') !== -1) {
			svgContainerWidth = chartContainerWidth * parseFloat(chart.width)/100;
		} else {
			svgContainerWidth = chart.width;
		}
		if (chart.height.indexOf('%') !== -1) {
			svgContainerHeight = chartContainerHeight * parseFloat(chart.height)/100;
		} else {
			svgContainerHeight = chart.width;
		}

		chartContainer.select("svg")
			.attr("width", svgContainerWidth)
			.attr("height", svgContainerHeight);

		axisDimensions = {
			left: 0,
			top: 0,
			right: 0,
			bottom: 0
		};
	}

	function drawTitle() {
		if (chart.title) {
			var drillUpButton = svgContainer.select(".drill-up-button");
			if (!drillUpButton.empty() && drillUpButton.style("display") !== "none") {
				drillUpButton = drillUpButton.select("rect").node().getBBox().width + 10;
			} else {
				drillUpButton = 0;
			}

			var titleContainer = svgContainer.select("g.title");
			if (titleContainer.empty()) {
				titleContainer = svgContainer.append("g")
					.attr("class", "title");
			}

			var title = titleContainer.select("text");
			if (title.empty()) {
				title = titleContainer.append("text")
					.attr("text-anchor","start")
					.style("font-size", chart.title.fontSize || "25px")
					.style("fill", chart.title.color || "#000");
			}
			title.text(chart.title.text || chart.title)			
				.attr("x", chart.padding.left + drillUpButton);

			var titleHeight = title.node().getBBox().height;
			title.attr("y", chart.padding.top + titleHeight);

			titlePadding = titleHeight + chart.padding.title;
		}
	}

	function drawSubTitle() {
		if (chart.subtitle) {
			var drillUpButton = svgContainer.select(".drill-up-button");
			if (!drillUpButton.empty() && drillUpButton.style("display") !== "none") {
				drillUpButton = drillUpButton.select("rect").node().getBBox().width + 10;
			} else {
				drillUpButton = 0;
			}

			var titleContainer = svgContainer.select("g.sub-title");
			if (titleContainer.empty()) {
				titleContainer = svgContainer.append("g")
					.attr("class", "sub-title");
			}

			var subtitle = titleContainer.select("text");
			if (subtitle.empty()) {
				subtitle = titleContainer.append("text")
					.attr("text-anchor","start")
					.style("font-size", chart.subtitle.fontSize || "15px")
					.style("fill", chart.subtitle.color || "#999");
			}
			subtitle.text(chart.subtitle.text || chart.subtitle)			
				.attr("x", chart.padding.left + drillUpButton);

			var titleHeight = subtitle.node().getBBox().height;
			subtitle.attr("y", chart.padding.top + titlePadding - chart.padding.title + titleHeight + 2);

			if (titlePadding === 0) {
				titlePadding += chart.padding.title;
			}
			titlePadding += titleHeight;
		}
	}

	function checkTitleOverflow() {
		var title = svgContainer.select("g.title text");
		var _text;
		if (!title.empty()) {
			_text = chart.title.text || chart.title;

			while (title.node().getBBox().width + parseFloat(title.attr("x")) > chart._chartWidth) {
				_text = _text.slice(0, -1);
				title.text(_text+"...");
			}
		}

		var subtitle = svgContainer.select("g.sub-title text");
		if (!subtitle.empty()){
			_text = chart.subtitle.text || chart.subtitle;
	
			while (subtitle.node().getBBox().width + parseFloat(subtitle.attr("x")) > chart._chartWidth) {
				_text = _text.slice(0, -1);
				subtitle.text(_text+"...");
			}
		}
	}

	var defaultSerieOptions = {
 			colors: ["#1f77b4","#ff7f0e","#2ca02c","#d62728","#9467bd","#8c564b","#e377c2","#7f7f7f","#bcbd22","#17becf"],
			yAxis: {
				position: "left",
				showGridLines: true,
				range: {}
			},
			showValueLabels: true 
		};
	var defaultBarOptions = {
			stacked: false,
			horizontal: false,
			floating: false,
			maxBarWidth: false,
			floatingGap: 0.1
		};
	var defaultLineOptions = {
			stacked: false,
			horizontal: false,
			lineType: "linear",
			lineWidth: 5,
			marks: {
				type: "circle",
				size: 10
			}
		};
	var defaultAreaOptions = jQuery.extend(true, {}, defaultLineOptions, {
			areaOpacity: 0.3,
			lineWidth: 0
		});
	var defaultPieOptions = {
			showValueLabels: "%",
			sort: false,
			strokeWidth: 0,
			strokeColor: "#FFFFFF",
			arcsGap: 0
		};
	var defaultDonutOptions = jQuery.extend(true, {}, defaultPieOptions, {
			donutWidth: 0.25
		});
	var defaultGaugeOptions = jQuery.extend(true, {}, defaultDonutOptions, {
			rangeType: "%",
			colors: ["#2ecc71","#f1c40f","#e67e22","#d35400"],
			gaugeWidth: 0.5,
			horizontal: false,
			pointerColor: "#000000",
			showGaugeValue: "value"
		});
	var defaultProgressOptions = {
			showValueLabels: false,
			showProgressValue: "%",
			backgroundColor: "#EEEEEE",
			progressWidth: 0.1,
			horizontal: false
		};
	var defaultSeriesOptions = {
			bar: defaultBarOptions,
			line: defaultLineOptions,
			area: defaultAreaOptions,
			pie: defaultPieOptions,
			donut: defaultDonutOptions,
			gauge: defaultGaugeOptions,
			progress: defaultProgressOptions
		};

	function drawBarSerie(serie) {
		var bars = serie._serieContainer._dataContainer.selectAll("g.bars").data(chart.dataSource.data);
		
		bars.transition().duration(_duration)
			.attr("transform", function(d) { 
				var translate;
				if (!chart.invertedAxes) {
					translate = "translate(" + chart._bandLeft(d) + ",0)"; 
				} else {
					translate = "translate(0," + serie._bandTop(d) + ")"; 
				}

				return translate;
			});

		bars.enter().append("g")
			.attr("class","bars")
			.attr("transform", function(data) {
				var translate;
				if (!chart.invertedAxes) {
					translate = "translate(" + chart._bandLeft(data) + ",0)"; 
				} else {
					translate = "translate(0," + serie._bandTop(data) + ")"; 
				}

				return translate;
			})
			.on("mouseover", showTooltip)
			.on("mousemove", showTooltip)
			.on("mouseout", hideTooltip);

		bars.exit().remove();

		var floatingScale;
		var floatingGap;
		if (serie.stacked && serie.floating) {
			if (!chart.invertedAxes) {
				floatingGap = serie.floatingGap * chart._chartHeight;
				floatingScale = d3.scale.linear().range([0, chart._chartHeight - (floatingGap * (serie.fields.length - 1))]);
			} else {			
				floatingGap = serie.floatingGap * chart._chartWidth;
				floatingScale = d3.scale.linear().range([0, chart._chartWidth - (floatingGap * (serie.fields.length - 1))]);
			}
			floatingScale.domain([0, d3.sum(serie.fields, function(field) {
				return serie._fieldsMaxValue[field];
			})]);
		}

		var barWidth, barHeight;
		if (!chart.invertedAxes) {
			if (serie.stacked) {
				barWidth = chart._bandWidth();
			} else {
				barWidth = chart._bandWidth()/serie.fields.length;
			}
		} else {
			if (serie.stacked) {
				barHeight = serie._bandHeight();
			} else {
				barHeight = serie._bandHeight()/serie.fields.length;
			}
		}

		function getBarColor(d, i) {
			var defaultColor = serie.colors[i % serie.colors.length];
			var value = d[serie.fields[i]];
			var color;
			if (!isUndefinedOrNull(serie.colorExpression)) {
				color = colorExpression(value, defaultColor, serie.colorExpression);
			} else {
				color = defaultColor;
			}

			return color;
		}

		function drawLabel(barGroup, fieldIndex, fieldName) {
			var label = barGroup.select("text.field-"+fieldIndex);

			if (!chart.invertedAxes) {
				var _barLeft = serie.stacked ? 0 : barWidth * fieldIndex;
				if (label.empty()) {
					label = barGroup.append("text")
						.attr("class", "label-value field-"+fieldIndex)
						.style("font-size", "11px")
						.attr("x", _barLeft + barWidth / 2)
						.attr("y", function(d) { return getSerieStartPosition(fieldIndex); })
						.attr("text-anchor","middle");
				}

				label.attr("dy", function(d) {
						var _dy;
						if (d[fieldName] < 0) {
							_dy = "1em";
						} else {
							_dy = "-0.3em";
						}
						return _dy;
					})
					.attr("fill", function(d) { return serie.stacked ? "#999" : getBarColor(d, fieldIndex); });

				label.transition().duration(_duration)
					.attr("x", _barLeft + barWidth / 2)
					.attr("y", function(d) {
						var _y;
						if (d[fieldName] < 0) {
							_y = serie._yScale(d[fieldName]);
						} else {
							_y = getSerieBarTop(d, fieldIndex);
						}
						return _y; 
					})
					.each("end", function(d, i) {
						var _label = d3.select(this);
						var labelBox = _label.node().getBBox();
						if (labelBox.y + labelBox.height > chart._chartHeight) {
							_label.attr("dy", "-0.3em")
								.attr("fill", "white");
						}
					});
			} else {
				var _barTop = serie.stacked ? 0 : barHeight * fieldIndex;
				if (label.empty()) {
					label = barGroup.append("text")
						.attr("class", "label-value field-"+fieldIndex)
						.style("font-size", "11px")
						.attr("x", function(d) { return getSerieStartPosition(fieldIndex); })
						.attr("y", _barTop + barHeight / 2)
						.attr("dy","0.35em");
				}

				label.attr("dx", function(d) {
						var _dx;
						if (d[fieldName] < 0) {
							_dx = "-0.3em";
						} else {
							_dx = "0.3em";
						}
						return _dx;
					})
					.attr("text-anchor", function(d) {
						var _ta;
						if (d[fieldName] < 0) {
							_ta = "end";
						} else {
							_ta = "start";
						}
						return _ta;
					})
					.attr("fill", function(d) { return serie.stacked ? "#999" : getBarColor(d, fieldIndex); });

				label.transition().duration(_duration)
					.attr("x", function(d) { 
						var _x;
						if (serie.stacked && serie.floating) {
							_x = getSerieBarLeft(d, fieldIndex) + getSerieBarWidth(d, fieldIndex); 
						} else {
							_x = chart._xScale(d[fieldName]); 
						}
						return _x;
					})
					.attr("y", _barTop + barHeight / 2)
					.each("end", function(d) {
						var _label = d3.select(this);
						var labelBox = _label.node().getBBox();
						
						if (serie.stacked && serie.floating && fieldIndex < serie.fields.length - 1) {
							if (labelBox.x + labelBox.width > getSerieStartPosition(fieldIndex+1)) {
								_label.attr("dx", "-0.71em")
									.attr("text-anchor", "end")
									.attr("fill", "white");
							}
						} else {
							if (labelBox.x + labelBox.width > chart._chartWidth) {
								_label.attr("dx", "-0.71em")
									.attr("text-anchor", "end")
									.attr("fill", "white");
							}
						}
						if (labelBox.x < 0) {
							_label.attr("dx", "0.71em")
								.attr("text-anchor", "start")
								.attr("fill", "white");
						}
					});

			}

			label.text(function(d) { 
				var value = d[fieldName];
				if (serie.yAxis.format) {
					if (isFunction(serie.yAxis.format)) {
						value = serie.yAxis.format(value);
					} else {
						value = formatValue(value, serie.yAxis.format);
					}
				}
				return value;
			});
		}

		function getSerieBarTop(d, i) {
			var barY;

			if (d[serie.fields[i]] < 0) {
				barY = chart._startPosition;
			} else if (serie.stacked && serie.floating) {
				barY = chart._startPosition - floatingScale(d[serie.fields[i]]);
			} else {
				barY = serie._yScale(d[serie.fields[i]]);
			}

			if (serie.stacked && i > 0) {
				for (var j = 0; j < i; j++) {
					if (!serie.floating) {
						barY = barY - (chart._startPosition - serie._yScale(d[serie.fields[j]]));
					} else {
						barY = barY - floatingScale(serie._fieldsMaxValue[serie.fields[j]]) - floatingGap;
					}
				}
			}
			return barY;
		}

		function getSerieBarLeft(d, i) {
			var barX;

			if (d[serie.fields[i]] < 0) {
				barX = chart._xScale(d[serie.fields[i]]);
			} else {
				barX = chart._startPosition;
			}

			if (serie.stacked && i > 0) {
				for (var j = 0; j < i; j++) {
					if (!serie.floating) {
						barX = barX + chart._xScale(d[serie.fields[j]]);
					} else {
						barX = barX + floatingScale(serie._fieldsMaxValue[serie.fields[j]]) + floatingGap;
					}
				}
			}
			return barX;
		}

		function getSerieStartPosition(i) {
			var pos = chart._startPosition;
			if (serie.stacked && serie.floating && i > 0) {
				for (var j = 0; j < i; j++) {
					if (!chart.invertedAxes) {
						pos = pos - floatingScale(serie._fieldsMaxValue[serie.fields[j]]) - floatingGap;
					} else {
						pos = pos + floatingScale(serie._fieldsMaxValue[serie.fields[j]]) + floatingGap;
					}
				}
			}

			return pos;
		}

		function getSerieBarHeight(d, i) {
			var height;
			if (serie.stacked && serie.floating) {
				height = floatingScale(d[serie.fields[i]]);
			} else {
				height = Math.abs(chart._startPosition - serie._yScale(d[serie.fields[i]]));
			}

			return height;
		}

		function getSerieBarWidth(d, i) {
			var width;
			if (serie.stacked && serie.floating) {
				width = floatingScale(d[serie.fields[i]]);
			} else {
				width = Math.abs(chart._xScale(d[serie.fields[i]]) - chart._startPosition);
			}
			return width;
		}

		serie.fields.forEach(function(field, i) {
			if (!chart.invertedAxes) {
				var _barLeft = serie.stacked ? 0 : barWidth * i;

				if (serie.stacked && serie.floating) {
					serie._yAxisContainer.select(".tick-"+i)
						.transition().duration(_duration)
						.attr("transform", "translate(0, " + getSerieStartPosition(i) + ")");
				}

				bars.each(function(d) {
					if (!isUndefinedOrNull(d[field])) {
						var barGroup = d3.select(this);
						var bar = barGroup.select("rect.field-"+i);

						var barOffset = 0;
						var _barWidth = barWidth;
						if (serie.maxBarWidth && barWidth > serie.maxBarWidth) {
							barOffset = (barWidth - serie.maxBarWidth) / 2;
							_barWidth = serie.maxBarWidth;
						}

						if (bar.empty()) {
							bar = barGroup.append("rect")
								.attr("class", "field-"+i)
								.attr("x", _barLeft + barOffset)
								.attr("y", function(d) { return getSerieStartPosition(i); })
								.attr("height", 0)
								.attr("width", _barWidth)
								.attr("fill", function(d) { return getBarColor(d, i); })
								.on("click", dataClick);
						}

						bar.transition().duration(_duration)
							.attr("x", _barLeft + barOffset)
							.attr("y", function(d) { return getSerieBarTop(d, i); })
							.attr("height", function(d) { return getSerieBarHeight(d, i); })
							.attr("fill", function(d) { return getBarColor(d, i); })
							.attr("width", _barWidth);
							
						if (serie.showValueLabels && (!serie.stacked || serie.floating)) {
							drawLabel(barGroup, i, serie.fields[i]);
						}
					}
				});
			} else {
				var _barTop = serie.stacked ? 0 : barHeight * i;

				if (serie.stacked && serie.floating) {
					chart._xAxisContainer.select(".tick-"+i)
						.transition().duration(_duration)
						.attr("transform", "translate(" + getSerieStartPosition(i) + ", 0)");
				}

				bars.each(function(d) {
					if (!isUndefinedOrNull(d[field])) {
						var barGroup = d3.select(this);
						var bar = barGroup.select("rect.field-"+i);

						var barOffset = 0;
						var _barHeight = barHeight;
						if (serie.maxBarWidth && barHeight > serie.maxBarWidth) {
							barOffset = (barHeight - serie.maxBarWidth) / 2;
							_barHeight = serie.maxBarWidth;
						}

						if (bar.empty()) {
							bar = barGroup.append("rect")
								.attr("class", "field-"+i)
								.attr("x", function(d) { return getSerieStartPosition(i); })
								.attr("y", _barTop + barOffset)
								.attr("height", _barHeight)
								.attr("width", 0)
								.attr("fill", function(d) { return getBarColor(d, i); })
								.on("click", dataClick);
						}

						bar.transition().duration(_duration)
							.attr("x", function(d) { return getSerieBarLeft(d, i); })
							.attr("y", _barTop + barOffset)
							.attr("height", _barHeight)
							.attr("fill", function(d) { return getBarColor(d, i); })
							.attr("width", function(d) { return getSerieBarWidth(d, i); });
							
						if (serie.showValueLabels && (!serie.stacked || serie.floating)) {
							drawLabel(barGroup, i, serie.fields[i]);
						}
					}
				});
			}
		});

		if (serie.stacked && !serie.floating && serie.showValueLabels) {
			bars.each(function() {
				var barGroup = d3.select(this);
				drawLabel(barGroup, serie.fields.length - 1, "_stackedValue");
			});
		}
	}

	function drawArea(serie) {
		serie.fields.forEach(function(field, i) {
			var initialArea = d3.svg.area()
				.interpolate(serie.lineType)
				.defined(function(d) { return !isUndefinedOrNull(d[field]); })
				.x0(function(d) { 
					var x;
					if (!chart.invertedAxes) {
						x = chart._pointLeft(d);
					} else {
						x = chart._startPosition;
					}
					return x; 
				})
				.x1(function(d) { 
					var x;
					if (!chart.invertedAxes) {
						x = chart._pointLeft(d);
					} else {
						x = chart._startPosition;
					}
					return x; 
				})
				.y0(function(d) { 
					var y;
					if (!chart.invertedAxes) {
						y = chart._startPosition;
					} else {
						y = serie._pointTop(d);
					}
					return y; 
				})
				.y1(function(d) { 
					var y;
					if (!chart.invertedAxes) {
						y = chart._startPosition;
					} else {
						y = serie._pointTop(d);
					}
					return y; 
				});

			var area = d3.svg.area()
				.interpolate(serie.lineType)
				.defined(function(d) { return !isUndefinedOrNull(d[field]); })
				.x0(function(d) { 
					var x;
					if (!chart.invertedAxes) {
						x = chart._pointLeft(d);
					} else {
						if (serie.stacked && i > 0) {
							x = chart._xScale(d["_stackedValue_"+serie.fields[i-1]]); 
						} else {
							x = chart._startPosition;
						}
					}
					return x; 
				})
				.x1(function(d) { 
					var x;
					if (!chart.invertedAxes) {
						x = chart._pointLeft(d);
					} else {
						if (serie.stacked) {
							x = chart._xScale(d["_stackedValue_"+field]); 
						} else {
							x = chart._xScale(d[field]);
						}
					}
					return x; 
				})
				.y0(function(d) { 
					var y; 
					if (!chart.invertedAxes) {
						if (serie.stacked && i > 0) {
							y = serie._yScale(d["_stackedValue_"+serie.fields[i-1]]); 
						} else {
							y = chart._startPosition;
						}
					} else {
						y = serie._pointTop(d);
					}
					return y;
				})
				.y1(function(d) { 
					var y; 
					if (!chart.invertedAxes) {
						if (serie.stacked) {
							y = serie._yScale(d["_stackedValue_"+field]); 
						} else {
							y = serie._yScale(d[field]); 
						}
					} else {
						y = serie._pointTop(d);
					}
					return y;
				});

			var path = serie._serieContainer._dataContainer.select("path.area.field-"+i);
			if (path.empty()) {
				path = serie._serieContainer._dataContainer.append("path")
					.attr("class", "area field-"+i)
					.attr("fill", serie.colors[i % serie.colors.length])
					.attr("opacity", serie.areaOpacity)
					.attr("stroke-width", 0)
					.attr("d", initialArea(chart.dataSource.data));
			}

			path.transition().duration(_duration)
				.attr("d", area(chart.dataSource.data));
		});
	}

	function drawLine(serie) {
		if (serie.lineType !== false) {
			serie.fields.forEach(function(field, i) {
				var initialLine = d3.svg.line()
					.interpolate(serie.lineType)
					.defined(function(d) { return !isUndefinedOrNull(d[field]); })
					.x(function(d) { 
						var _x;
						if (!chart.invertedAxes) {
							_x = chart._pointLeft(d);
						} else {
							_x = chart._startPosition;
						}
						return _x; 
					})
					.y(function(d) { 
						var _y;
						if (!chart.invertedAxes) {
							_y = chart._startPosition;
						} else {
							_y = serie._pointTop(d);
						}
						return _y; 
					});

				var line = d3.svg.line()
					.interpolate(serie.lineType)
					.defined(function(d) { return !isUndefinedOrNull(d[field]); })
					.x(function(d) { 
						var _x;
						if (!chart.invertedAxes) {
							_x = chart._pointLeft(d);
						} else {
							if (serie.stacked) {
								_x = chart._xScale(d["_stackedValue_"+field]); 
							} else {
								_x = chart._xScale(d[field]); 
							}
						}
						return _x; 
					})
					.y(function(d) { 
						var _y; 
						if (!chart.invertedAxes) {
							if (serie.stacked) {
								_y = serie._yScale(d["_stackedValue_"+field]); 
							} else {
								_y = serie._yScale(d[field]); 
							}
						} else {
							_y = serie._pointTop(d);
						}

						return _y;
					});

				var path = serie._serieContainer._dataContainer.select("path.line.field-"+i);
				if (path.empty()) {
					path = serie._serieContainer._dataContainer.append("path")
						.attr("class", "line field-"+i)
						.attr("fill", "none")
						.attr("stroke-width", serie.lineWidth)
						.attr("stroke-linecap", "round")
						.attr("stroke-linejoin", "round")
						.attr("d", initialLine(chart.dataSource.data))
						.attr("stroke", serie.colors[i % serie.colors.length]);
				}

				path.transition().duration(_duration)
					.attr("d", line(chart.dataSource.data));
			});
		}
	}

	function drawMarks(serie) {
		var getMarkY = function(field, d) {
			var y;
			if (serie.stacked) {
				y = serie._yScale(d["_stackedValue_"+field]);
			} else {
				y = serie._yScale(d[field]);					
			}
			return y;
		};

		var getMarkX = function(field, d) {
			var x;
			if (serie.stacked) {
				x = chart._xScale(d["_stackedValue_"+field]);
			} else {
				x = chart._xScale(d[field]);					
			}
			return x;
		};

		var getTextAnchor = function(d, i, field) {
			var leftDir, rightDir;
			var leftValue, rightValue, value;
			var fieldName = serie.stacked ? "_stackedValue_"+field : field;

			var textAnchor;
			if (i === 0) {
				textAnchor = "start";
			} else {
				value = chart.dataSource.data[i][fieldName];
				leftValue = chart.dataSource.data[i - 1][fieldName];

				leftDir = leftValue > value ? "up" : "down";
			}

			if (i == chart.dataSource.data.length - 1) {
				textAnchor = "end";
			} else {
				value = chart.dataSource.data[i][fieldName];
				rightValue = chart.dataSource.data[i + 1][fieldName];

				rightDir = rightValue > value ? "up" : "down";
			}

			return textAnchor || (leftDir != rightDir ? "start" : "middle");
		};

		var getTextAnchorInverted = function(d, i, field) {
			var fieldName = serie.stacked ? "_stackedValue_"+field : field;
			var textAnchor;
			if (i === 0) {
				textAnchor = "start";
			} else {
				textAnchor = chart.dataSource.data[i - 1][fieldName] <= chart.dataSource.data[i][fieldName] ? "start" : "end";
			}

			return textAnchor;
		};

		var getTextRelativeY = function(d, i, field) {
			var fieldName = serie.stacked ? "_stackedValue_"+field : field;
			var textRelativeY;
			if (i == chart.dataSource.data.length - 1) {
				textRelativeY = chart.dataSource.data[i - 1][fieldName] > chart.dataSource.data[i][fieldName] ? "1.7em" : "-1em";
			} else {
				textRelativeY = chart.dataSource.data[i + 1][fieldName] > chart.dataSource.data[i][fieldName] ? "1.7em" : "-1em";
			}

			return textRelativeY;
		};

		var getTextRelativeX = function(d, i, field) {
			var fieldName = serie.stacked ? "_stackedValue_"+field : field;
			var textRelativeY;
			if (i === 0) {
				textRelativeY = "0.6em";
			} else {
				textRelativeY = chart.dataSource.data[i - 1][fieldName] <= chart.dataSource.data[i][fieldName] ? "0.6em" : "-0.6em";
			}

			return textRelativeY;
		};

		if (serie.marks) {
			var marks = serie._serieContainer._dataContainer.selectAll("g.marks").data(chart.dataSource.data);

			marks.enter().append("g")
				.attr("class","marks");

			marks.exit().remove();

			serie.fields.forEach(function(field, i) {
				var symbol;
				marks.each(function(markData, markIndex) {
					if (!isUndefinedOrNull(markData[field])) {
						var markGroup = d3.select(this);
						var markY = getMarkY(field, markData);
						var markX = getMarkX(field, markData);

						if (serie.marks.type !== false) {
							var mark = markGroup.select("path.field-"+i);
							if (mark.empty()) {
								symbol = d3.svg.symbol().type(serie.marks.type).size(0);
								mark = markGroup.append("path")
									.attr("class","field-"+i)
									.attr("transform", function(d) { 
										if (!chart.invertedAxes) {
											this._translate = "translate(" + chart._pointLeft(d) + ", " + chart._startPosition + ")";
										} else {
											this._translate = "translate(" + chart._startPosition + ", " + serie._pointTop(d) + ")";
										}
										return this._translate;
									})
									.attr("d", symbol)
									.attr("fill", serie.colors[i % serie.colors.length])
									.on("mouseover", function(d) {
										d3.select(this).transition().duration(100).attr("transform", this._translate + " scale(1.5)");
										showTooltip(d); 
									})
									.on("mousemove", showTooltip)
									.on("mouseout", function(d) {
										d3.select(this).transition().duration(100).attr("transform", this._translate + " scale(1)");
										hideTooltip(d); 
									})
									.on("mouseup", dataClick);
							}
							symbol = d3.svg.symbol().type(serie.marks.type).size(Math.pow(serie.marks.size,2));

							if (!chart.invertedAxes) {
								mark.transition().duration(_duration)
									.attr("transform", function(d, i) { 
										this._translate = "translate(" + chart._pointLeft(d) + ", " + markY + ")";
										return this._translate; 
									})
									.attr("d", symbol);
							} else {
								mark.transition().duration(_duration)
									.attr("transform", function(d, i) { 
										this._translate = "translate(" + markX + ", " + serie._pointTop(d) + ")";
										return this._translate; 
									})
									.attr("d", symbol);
							}
						}

						if (serie.showValueLabels) {
							var label = markGroup.select("text.field-"+i);

							if (!chart.invertedAxes) {
								if (label.empty()) {
									label = markGroup.append("text")
										.attr("class", "label-value field-"+i)
										.style("font-size", "11px")
										.attr("x", function(d) { return chart._pointLeft(d); })
										.attr("y", chart._startPosition)
										.attr("dy", function(d) { return getTextRelativeY(d, markIndex, field); })
										.attr("text-anchor", function(d) { return getTextAnchor(d, markIndex, field); })
										.attr("fill", serie.colors[i % serie.colors.length])
										.on("mouseover", showTooltip)
										.on("mousemove", showTooltip)
										.on("mouseout", hideTooltip)
										.on("mouseup", dataClick);
								}

								label.transition().duration(_duration)
									.attr("x", function(d) { return chart._pointLeft(d); })
									.attr("y", function(d) { return markY; });				
							} else {
								if (label.empty()) {
									label = markGroup.append("text")
										.attr("class", "label-value field-"+i)
										.style("font-size", "11px")
										.attr("x", chart._startPosition)
										.attr("y", function(d) { return serie._pointTop(d); })
										.attr("dy", "-0.6em")
										.attr("dx", function(d) { return getTextRelativeX(d, markIndex, field); })
										.attr("text-anchor", function(d) { return getTextAnchorInverted(d, markIndex, field); })
										.attr("fill", serie.colors[i % serie.colors.length])
										.on("mouseover", showTooltip)
										.on("mousemove", showTooltip)
										.on("mouseout", hideTooltip)
										.on("mouseup", dataClick);
								}

								label.transition().duration(_duration)
									.attr("x", function(d) { return markX; })				
									.attr("y", function(d) { return serie._pointTop(d); })
									.each("end", function(d, i) {
										var _label = d3.select(this);
										var labelBox = _label.node().getBBox();
										if (labelBox.x + labelBox.width > chart._chartWidth) {
											_label.attr("dx", "-0.6em")
												.attr("text-anchor", "end");
										}
										if (labelBox.x < 0) {
											_label.attr("dx", "0.6em")
												.attr("text-anchor", "start");
										}
									});
							}

							label.text(function(d) { 
								var value = d[field];
								if (serie.yAxis.format) {
									if (isFunction(serie.yAxis.format)) {
										value = serie.yAxis.format(value);
									} else {
										value = formatValue(value, serie.yAxis.format);
									}
								}
								return value;
							});
							
						}
					}
				});
			});
		}
	}

	function drawLineSerie(serie) {
		drawLine(serie);
		drawMarks(serie);
	}

	function drawAreaSerie(serie) {
		drawArea(serie);
		drawLine(serie);
		drawMarks(serie);
	}

	function checkLabelOverlap(labels) {
		var again = false;
		var a, b;
		var minSpace = 15;

		labels.each(function(aData, i) {
			a = this;

			labels.each(function(bData, j) {
				b = this;

				if (a == b || i > j) return; // same object

				var aObj = d3.select(a);
				var bObj = d3.select(b);

				if (aObj.attr("text-anchor") != bObj.attr("text-anchor")) return; // diferent text-anchor don't collide

				var aPosY = parseFloat(aObj.attr("new-y") || aObj.attr("transform").substring(10).slice(0,-1).split(", ")[1]);
				var bPosY = parseFloat(bObj.attr("new-y") || bObj.attr("transform").substring(10).slice(0,-1).split(", ")[1]);
				var delta = aPosY - bPosY;

				if (Math.round(Math.abs(delta)) >= minSpace) return; // no collision

				again = true;

				var aAngle = (aData.startAngle + aData.endAngle) / 2;
				var signY = aObj.attr("text-anchor") == "end" ? 1 : -1;
				var alpha = (minSpace - Math.abs(delta)) * signY;

				if (Math.tan(aAngle) > 0) { // first and third quadrant
					aObj.attr("new-y", aPosY + alpha);
				} else {
					bObj.attr("new-y", bPosY - alpha);
				}
			});
		});

		if (again) {
			checkLabelOverlap(labels);
		} else {
			labels.each(function(data) {
				a = d3.select(this);

				if (a.attr("new-y")) {
					var posX = a.attr("transform").substring(10).slice(0,-1).split(", ")[0];
					var posY = a.attr("transform").substring(10).slice(0,-1).split(", ")[1];

					a.transition().duration(250).attr("transform", "translate(" + posX + ", " + a.attr("new-y") + ")");
					a.attr("new-y", null);
				}
			});
		}
	}

	function getLabelValue(serie, d) {
		var value;

		if (serie.showValueLabels == "value") {
			value = d.data[serie.fieldValue];
			if (serie.valueFormat) {
				if (isFunction(serie.valueFormat)) {
					value = serie.valueFormat(value);
				} else {
					value = formatValue(value, serie.valueFormat);
				}
			}
		} else if (serie.showValueLabels == "%") {
			value = d.data[serie.fieldValue];
			value = (value * 100 / serie._sumData).formatFloat(2)+"%";
		} else if (serie.showValueLabels == "label") {
			value = d.data[serie.fieldLabel];
		}

		return value;
	}
	
	var _labelRadius = 20;
	function drawSlices(serie, radius) {
		var pie, pieData, dataLength, slicesY;
		if (serie.type == "gauge") {
			pie = d3.layout.pie()
				.sort(null)
				.value(function(d) { return d.rangeSize; })
				.startAngle(-Math.PI/2)
				.endAngle(Math.PI/2)
				.padAngle(serie.arcsGap);

			pieData = pie(serie._gaugeData);
			dataLength = serie._gaugeData.length;
			slicesY = (chart._chartHeight + radius) / 2;
		} else {
			pie = d3.layout.pie()
				.sort(null)
				.value(function(d) { return d[serie.fieldValue]; })
				.padAngle(serie.arcsGap);

			pieData = pie(serie.pieData);
			dataLength = serie.pieData.length;
			slicesY = chart._chartHeight/2;
		}

		var slices = serie._serieContainer._dataContainer.selectAll("g.slices");
		if (slices.empty()) {
			slices = serie._serieContainer._dataContainer.append("g")
				.attr("class", "slices")
				.attr("transform", "translate(" + chart._chartWidth/2 + ", " + slicesY + ")");
		}

		slices.transition().duration(_duration)
			.attr("transform", "translate(" + chart._chartWidth/2 + ", " + slicesY + ")");

		var arcs = slices.selectAll("path").data(pieData);
		var arcsLength = arcs.size();

		arcs.enter().append("path")
			.attr("class","slice")
			.attr("fill", function(d, i) { return serie.colors[i % serie.colors.length]; })
			.attr("stroke", serie.strokeColor)
			.attr("stroke-width", serie.strokeWidth)
			.on("mouseover", showTooltip)
			.on("mousemove", showTooltip)
			.on("mouseout", hideTooltip)
			.on("mouseup", function(d) {
				dataClick(d.data); 
			});

		arcs.exit().remove();

		arcs.transition().duration(_duration)
			.attrTween("d", function(d) {
				if (isUndefinedOrNull(this._current)) {
					this._current = jQuery.extend(true, {}, d);
					if (serie.type == "gauge") {
						this._current.startAngle = -Math.PI/2;
						this._current.endAngle = -Math.PI/2;
						this._current.radius = radius;
					} else {
						if (arcsLength > 0) {
							this._current.startAngle = Math.PI*2;
							this._current.endAngle = Math.PI*2;
							this._current.radius = radius;
						} else {
							this._current.startAngle = 0;
							this._current.endAngle = 0;
							this._current.radius = 0;
						}
					}
				}
				var dataInterpolation = d3.interpolate(this._current, d);
				var radiusInterpolation = d3.interpolate(this._current.radius, radius);
				this._current = d;
				this._current.radius = radius;

				return function(t) {
					d = dataInterpolation(t);

					var r = radiusInterpolation(t);
					var arc = d3.svg.arc().outerRadius(r);
					
					if (serie.type == "donut" || serie.type == "gauge") {
						arc.innerRadius(r - r * serie.donutWidth);
					} else {
						arc.innerRadius(0);
					}

					return arc(d);
				};
			});

		if (serie.showValueLabels !== false && serie.type != "gauge") {
			var labels = slices.selectAll("text.label-value").data(pieData);

			labels.enter().append("text")
				.attr("class", "label-value")
				.attr("fill", function(d, i) { return serie.colors[i % serie.colors.length]; })
				.style("font-size", "11px")
				.attr("dy", "0.35em")
				.on("mouseover", showTooltip)
				.on("mousemove", showTooltip)
				.on("mouseout", hideTooltip)
				.attr("text-anchor", function(d, i) {
					var arc = d3.svg.arc()
							.outerRadius(radius)
							.innerRadius(radius);

					var centroid = arc.centroid(d);
					var midAngle = Math.atan2(centroid[1], centroid[0]);
					var x = Math.cos(midAngle) * _labelRadius;
					return x > 0 ? "start" : "end";
				});

			labels.exit().remove();

			labels.text(function(d) { return getLabelValue(serie, d); });

			labels.transition().duration(_duration)
				.each("end", function(d, i) {
					if (i == dataLength - 1) {
						checkLabelOverlap(labels);
					}
				})
				.attrTween("transform", function(d) {
					if (isUndefinedOrNull(this._current)) {
						this._current = jQuery.extend(true, {}, d);

						if (arcsLength > 0) {
							this._current.startAngle = Math.PI*2;
							this._current.endAngle = Math.PI*2;
							this._current.radius = radius;
						} else {
							this._current.startAngle = 0;
							this._current.endAngle = 0;
							this._current.radius = 0;
						}
					}
					var dataInterpolation = d3.interpolate(this._current, d);
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					this._current = d;
					this._current.radius = radius;

					return function(t) {
						d = dataInterpolation(t);

						var r = radiusInterpolation(t);
						var arc = d3.svg.arc()
							.outerRadius(r)
							.innerRadius(r);

						var centroid = arc.centroid(d);
						var x = centroid[0], y = centroid[1];
						var midAngle = Math.atan2(y, x);

						var _x = Math.cos(midAngle) * _labelRadius/2;
						var sign = (_x > 0) ? 1 : -1;
						x += _x + (5 * sign);

						var _y = Math.sin(midAngle) * _labelRadius/2;
						y += _y;

						return "translate(" + x + ", " + y + ")";
					};
				});
		}
	}

	function getPieLabelsMaxDim(serie) {
		var dim = {
			width: 0,
			height: 0
		};

		var text, value;
		for (var i = 0; i < serie.pieData.length; i++) {
			value = getLabelValue(serie, { data: serie.pieData[i] });

			text = serie._serieContainer.append("text")
				.attr("font-size", "11px")
				.text(value);

			dim.width = Math.max(dim.width, text.node().getBBox().width);
			dim.height = Math.max(dim.height, text.node().getBBox().height);

			text.remove();
		}

		return dim;
	}

	function drawPieDonutSerie(serie) {
		var radius = Math.min(chart._chartWidth, chart._chartHeight) / 2;
		if (serie.showValueLabels !== false) {
			var labelsMaxDim = getPieLabelsMaxDim(serie);
			radius = Math.min(chart._chartWidth - labelsMaxDim.width * 2, chart._chartHeight - labelsMaxDim.height * 2) / 2 - _labelRadius;
		}

		drawSlices(serie, radius);
	}

	function drawGaugePointer(serie, radius) {
		var config, pointerX, pointerY;
		var startX, maxWidth;

		if (serie.horizontal) {
			config = {
				pointerWidth: Math.round(radius * 0.03),
				pointerTailLength: Math.round(radius * 0.02),
				pointerHeadLength: Math.round(radius * serie.gaugeWidth * 0.95)
			};
			pointerX = chart._chartWidth * 0.1;
			pointerY = radius * (1 + serie.gaugeWidth) / 2;
		} else {
			config = {
				pointerWidth: Math.round(radius * 0.05),
				pointerTailLength: Math.round(radius * 0.03),
				pointerHeadLength: Math.round(radius * 0.95)
			};
			pointerX = chart._chartWidth/2;
			pointerY = (chart._chartHeight + radius) / 2;
		}

		var lineData = [ 
			[config.pointerWidth / 2, 0], 
			[0, -config.pointerHeadLength],
			[-(config.pointerWidth / 2), 0],
			[0, config.pointerTailLength]
		];		
		var pointerLine = d3.svg.line().interpolate('monotone');

		var pointer = serie._serieContainer._dataContainer.selectAll("g.pointer");
		if (pointer.empty()) {
			pointer = serie._serieContainer._dataContainer.append("g").data([lineData])
				.attr("class", "pointer")
				.attr("fill", serie.pointerColor)
				.attr("transform", "translate(" + pointerX + ", " + pointerY + ")");

			pointer.append("path");

			if (!serie.horizontal) {
				pointer.select("path").attr("transform", "rotate(-90)");
			}
		}

		var angle;
		if (serie.horizontal) {
			startX = chart._chartWidth * 0.1;
			maxWidth = chart._chartWidth * 0.8;
			angle = 0;
			pointerX = startX + maxWidth * serie.value / (serie._maxValue - serie._minValue);
		} else {
			angle = Math.min(-90 + (180 * serie.value / (serie._maxValue - serie._minValue)), 90);
		}

		pointer.data([lineData]).transition().duration(_duration)
			.attr("transform", "translate(" + pointerX + ", " + pointerY + ")")
			.select("path")
			.attr("d", function(d) { return pointerLine(d) + "Z"; })
			.attr("transform", "rotate(" + angle + ")");			

		if (serie.showValueLabels) {
			var label = pointer.selectAll("text.label-value");
			
			if (label.empty()) {
				label = pointer.append("text")
					.attr("class", "label-value")
					.style("font-size", "11px")
					.attr("dy", "0.35em");
			}

			if (serie.horizontal) {
				label.attr("text-anchor", "middle");
			}

			label.transition().duration(_duration)
				.each("end", function(d) {
					var obj = d3.select(this);
					if (serie.showValueLabels == "%"){
						obj.text((serie.value * 100 / (serie._maxValue - serie._minValue)).formatFloat(2)+"%");
					} else {
						var value = serie.value;
						if (serie.valueFormat) {
							if (isFunction(serie.valueFormat)) {
								value = serie.valueFormat(value);
							} else {
								value = formatValue(value, serie.valueFormat);
							}
						}
						obj.text(value);
					}
				})
				.attrTween("transform", function(d) {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { angle: -90, radius: radius, value: serie._minValue };
					}
					var angleInterpolation = d3.interpolate(this._current.angle, angle);
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					var valueInterpolation = d3.interpolate(this._current.value, serie.value);
					this._current.angle = angle;
					this._current.radius = radius;
					this._current.value = serie.value;

					return function(t) {
						var x, y, percent;
						var aRad;
						var r = radiusInterpolation(t);

						if (serie.horizontal) {
							var v = valueInterpolation(t);
							percent = v * 100 / (serie._maxValue - serie._minValue);

							x = 0;
							y = (-r * serie.gaugeWidth) - (_labelRadius / 2);
						} else {						
							var a = angleInterpolation(t);
							aRad = a * Math.PI / 180;
						
							percent = (a + 90) * 100 / 180;

							x = Math.sin(aRad) * (r + _labelRadius / 2);
							y = Math.cos(aRad) * (r + _labelRadius / 2) * -1;
						}

						var anchor;
						if (serie.horizontal) {
							anchor = "middle";
						} else {							
							if (Math.tan(aRad) < 0) {
								anchor = "end";	
							} else if (Math.tan(aRad) === 0) {
								anchor = "middle";
							} else {
								anchor = "start";
							}
						}
						obj.attr("text-anchor", anchor);
						
						if (serie.showValueLabels == "%") {
							obj.text(percent.formatFloat(2)+"%");
						} else {
							var value = (serie._maxValue - serie._minValue) * percent / 100;
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									value = serie.valueFormat(value);
								} else {
									value = formatValue(value, serie.valueFormat);
								}
							}
							obj.text(value);
						}

						return "translate(" + x + ", " + y + ")";
					};
				});
		}

		if (serie.showGaugeValue) {
			var valueLabel = pointer.selectAll("text.value");
			
			if (valueLabel.empty()) {
				valueLabel = pointer.append("text")
					.attr("class", "value")
					.attr("text-anchor", "middle")
					.attr("font-size", 21)
					.attr("dy", "2em");

				if (serie.horizontal) {
					valueLabel.attr("dy", "2.5em");
				}
			}

			valueLabel.transition().duration(_duration)
				.tween("text", function() {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { value: 0 };
					}
					var valueInterpolation = d3.interpolate(this._current.value, serie.value);
					this._current.value = serie.value;

					return function(t) {
						var v = valueInterpolation(t);

						if (serie.showGaugeValue == "%") {
							obj.text((v * 100 / serie.maxValue).formatFloat(2)+"%");
						} else {
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									v = serie.valueFormat(v);
								} else {
									v = formatValue(v, serie.valueFormat);
								}
							}
							obj.text(v);
						}
					};
				});
		}
	}

	function drawGaugeLimits(serie, radius) {
		var limitsX, limitsY;

		if (serie.horizontal) {
			limitsX = chart._chartWidth * 0.1;
			limitsY = chart._chartHeight * (1 + serie.gaugeWidth) / 2;
		} else {
			limitsX = chart._chartWidth / 2;
			limitsY = (chart._chartHeight + radius) / 2;
		}

		var limits = serie._serieContainer._dataContainer.selectAll("g.limits");
		if (limits.empty()) {
			limits = serie._serieContainer._dataContainer.append("g")
				.attr("class", "limits")
				.attr("transform", "translate(" + limitsX + ", " + limitsY + ")");
		}

		limits.transition().duration(_duration)
			.attr("transform", "translate(" + limitsX + ", " + limitsY + ")");

		var minLimit = limits.select(".min-limit");
		var maxLimit = limits.select(".max-limit");

		var minLimitX, maxLimitX;

		if (serie.horizontal) {
			minLimitX = 0;
			maxLimitX = chart._chartWidth * 0.8;
		} else {
			minLimitX = radius*(-1+serie.donutWidth/2);
			maxLimitX = radius*(1-serie.donutWidth/2);
		}

		var fontSize = 14;
		if (minLimit.empty()) { // && maxLimit.empty()
			minLimit = limits.append("text")
				.attr("class", "min-limit")
				.attr("font-size", fontSize)
				.attr("text-anchor", "middle")
				.attr("dy", "1.3em")
				.attr("transform", "translate(" + minLimitX + ", 0)");

			maxLimit = limits.append("text")
				.attr("class", "max-limit")
				.attr("font-size", fontSize)
				.attr("text-anchor", "middle")
				.attr("dy", "1.3em")
				.attr("transform", "translate(" + maxLimitX + ", 0)");
		}

		minLimit.text(function() {
			var value = serie._minValue;
			if (serie.valueFormat) {
				if (isFunction(serie.valueFormat)) {
					value = serie.valueFormat(value);
				} else {
					value = formatValue(value, serie.valueFormat);
				}
			}
			return value;
		});

		maxLimit.text(function() {
			var value = serie._maxValue;
			if (serie.valueFormat) {
				if (isFunction(serie.valueFormat)) {
					value = serie.valueFormat(value);
				} else {
					value = formatValue(value, serie.valueFormat);
				}
			}
			return value;
		});

		if (serie.horizontal) {
			while (maxLimit.node().getBBox().width >= (chart._chartWidth * 0.2)) {
				fontSize--;
				limits.selectAll("text").attr("font-size", fontSize);
			}
		} else {
			while (maxLimit.node().getBBox().width >= (radius * serie.donutWidth)) {
				fontSize--;
				limits.selectAll("text").attr("font-size", fontSize);
			}
		}
		
		minLimit.transition().duration(_duration)
			.attr("transform", "translate(" + minLimitX + ", 0)");

		maxLimit.transition().duration(_duration)
			.attr("transform", "translate(" + maxLimitX + ", 0)");
	}

	function drawGaugeBands(serie) {
		var bands = serie._serieContainer._dataContainer.selectAll("g.bands");
		if (bands.empty()) {
			bands = serie._serieContainer._dataContainer.append("g")
				.attr("class", "bands")
				.attr("transform", "translate(" + chart._chartWidth * 0.1 + ", " + chart._chartHeight/2 + ")");
		}

		bands.transition().duration(_duration)
			.attr("transform", "translate(" + chart._chartWidth * 0.1 + ", " + chart._chartHeight/2 + ")");

		var rects = bands.selectAll("rect").data(serie._gaugeData);

		var maxWidth = chart._chartWidth * 0.8;
		rects.enter().append("rect")
			.attr("class","band")
			.attr("fill", function(d, i) { return serie.colors[i % serie.colors.length]; })
			.attr("stroke", serie.strokeColor)
			.attr("stroke-width", serie.strokeWidth)
			.attr("width", 0)
			.attr("height", chart._chartHeight * serie.gaugeWidth)
			.attr("x", 0)
			.attr("y", chart._chartHeight * -serie.gaugeWidth / 2)
			.on("mouseover", showTooltip)
			.on("mousemove", showTooltip)
			.on("mouseout", hideTooltip)
			.on("mouseup", dataClick);

		rects.exit().remove();

		function getBandWidth(value) {
			return value * maxWidth / serie._maxValue;
		}

		rects.transition().duration(_duration)
			.attr("height", chart._chartHeight * serie.gaugeWidth)
			.attr("width", function(d, i) { return getBandWidth(d.rangeSize); })
			.attr("x", function(d, i) { return i === 0 ? 0 : getBandWidth(serie.ranges[i-1]) + getBandWidth(serie._gaugeData[i-1].rangeSize); })
			.attr("y", chart._chartHeight * -serie.gaugeWidth / 2);
	}

	function getGaugeLabelsMaxDim(serie) {
		var dim = {
			width: 0,
			height: 0
		};

		var text = serie._serieContainer.append("text")
			.attr("font-size", "11px");

		if (serie.showValueLabels == "%") {
			text.text((100).formatFloat(2)+"%");
		} else {
			var value = serie._maxValue;
			if (serie.valueFormat) {
				if (isFunction(serie.valueFormat)) {
					value = serie.valueFormat(value);
				} else {
					value = formatValue(value, serie.valueFormat);
				}
			}
			text.text(value);
		}

		dim.width = Math.max(dim.width, text.node().getBBox().width);
		dim.height = Math.max(dim.height, text.node().getBBox().height);

		text.remove();

		return dim;
	}

	function drawGaugeSerie(serie) {
		var valueHeight = serie.showGaugeValue ? 30 : 0;
		var radius = Math.min(chart._chartWidth / 2, chart._chartHeight - valueHeight - 10);
		if (serie.showValueLabels !== false) {
			var labelsMaxDim = getGaugeLabelsMaxDim(serie);
			radius = Math.min(chart._chartWidth / 2 - labelsMaxDim.width, chart._chartHeight - valueHeight - 10 - labelsMaxDim.height) - _labelRadius;
		}

		if (serie.horizontal) {
			radius = chart._chartHeight;
			drawGaugeBands(serie);
		} else {
			serie.donutWidth = serie.gaugeWidth;
			drawSlices(serie, radius);
		}

		drawGaugeLimits(serie, radius);
		drawGaugePointer(serie, radius);
	}

	function drawProgressSerie(serie) {
		var radius;
		var progressX, progressY, progressWidth, maxWidth;

		if (serie.horizontal) {
			radius = chart._chartHeight;
			progressX = chart._chartWidth * 0.1;
		} else {
			radius = Math.min(chart._chartWidth, chart._chartHeight) / 2;
			if (serie.showValueLabels !== false) {
				radius -= _labelRadius;
			}

			progressX = chart._chartWidth / 2;
		}
		progressY = chart._chartHeight / 2;
		progressWidth = chart._chartHeight * serie.progressWidth;
		maxWidth = chart._chartWidth * 0.8;

		var progressContainer = serie._serieContainer._dataContainer.selectAll("g.progress-container");
		if (progressContainer.empty()) {
			progressContainer = serie._serieContainer._dataContainer.append("g")
				.attr("class", "progress-container")
				.attr("transform", "translate(" + progressX + ", " + progressY + ")");
		}
		progressContainer.transition().duration(_duration)
			.attr("transform", "translate(" + progressX + ", " + progressY + ")");

		progressBackground = progressContainer.select(".progress-background");
		if (progressBackground.empty()) {
			if (serie.horizontal) {
				progressBackground = progressContainer.append("rect")
					.attr("x", 0)
					.attr("y", -progressWidth / 2)
					.attr("rx", progressWidth / 2)
					.attr("ry", progressWidth / 2)
					.attr("width", maxWidth)
					.attr("height", progressWidth);
					
			} else {
				progressBackground = progressContainer.append("path")
					.attr("d", function(d) {
						var arc = d3.svg.arc()
							.innerRadius(radius * (1 - serie.progressWidth))
							.outerRadius(radius)
							.startAngle(0)
							.endAngle(Math.PI * 2);
						return arc();
					});
			}

			progressBackground.attr("class","progress-background")
				.attr("stroke-width",0)
				.attr("fill", serie.backgroundColor);
		}


		if (serie.horizontal) {
			progressBackground.transition().duration(_duration)
				.attr("x", 0)
				.attr("y", -progressWidth / 2)
				.attr("rx", progressWidth / 2)
				.attr("ry", progressWidth / 2)
				.attr("width", maxWidth)
				.attr("height", progressWidth);
		} else {
			progressBackground.transition().duration(_duration)
				.attr("d", function(d) {
					var arc = d3.svg.arc()
						.innerRadius(radius * (1 - serie.progressWidth))
						.outerRadius(radius)
						.startAngle(0)
						.endAngle(Math.PI * 2);
					return arc();
				});
		}

		progress = progressContainer.select(".progress-indicator");
		if (progress.empty()) {
			if (serie.horizontal) {
				progress = progressContainer.append("rect")
					.attr("x", 0)
					.attr("y", -progressWidth / 2)
					.attr("rx", progressWidth / 2)
					.attr("ry", progressWidth / 2)
					.attr("width", 0)
					.attr("height", progressWidth);
			} else {
				progress = progressContainer.append("path");
			}

			progress.attr("class","progress-indicator")
				.attr("stroke-width",0)
				.attr("fill", serie.colors[0]);
		}

		if (serie.horizontal) {
			progress.transition().duration(_duration)
				.attr("y", -progressWidth / 2)
				.attr("rx", progressWidth / 2)
				.attr("ry", progressWidth / 2)
				.attr("width", serie.value * maxWidth / serie.maxValue)
				.attr("height", progressWidth);
		} else {
			progress.transition().duration(_duration)
				.attrTween("d", function() {
					if (isUndefinedOrNull(this._current)) {
						this._current = { endAngle: 0, radius: radius };
					}
					var angle = Math.PI * 2 * serie.value / serie.maxValue;
					var angleInterpolation = d3.interpolate(this._current.endAngle, angle);
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					this._current.endAngle = angle;
					this._current.radius = radius;

					return function(t) {
						var a = angleInterpolation(t);
						var r = radiusInterpolation(t);

						var arc = d3.svg.arc()
							.innerRadius(r * (1 - serie.progressWidth))
							.outerRadius(r)
							.cornerRadius(r * serie.progressWidth)
							.startAngle(0)
							.endAngle(a);
						
						return arc();
					};
				});
		}

		if (serie.showProgressValue) {
			var valueLabel = progressContainer.selectAll("text.value");
			
			if (valueLabel.empty()) {
				valueLabel = progressContainer.append("text")
					.attr("class", "value")
					.attr("text-anchor", "middle")
					.attr("dy", "0.35em");

				if (serie.horizontal) {
					valueLabel.attr("dy", "1em");
				}
			}

			valueLabel.transition().duration(_duration)
				.attrTween("font-size", function() {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { radius: radius, value: 0, maxWidth: maxWidth };
					}
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					var valueInterpolation = d3.interpolate(this._current.value, serie.value);
					var maxWidthInterpolation = d3.interpolate(this._current.maxWidth, maxWidth);
					this._current.radius = radius;
					this._current.value = serie.value;
					this._current.maxWidth = maxWidth;

					return function(t) {
						var objWidth = obj.node().getBBox().width;
						var fontSize = parseInt(obj.attr("font-size") || 45);

						var r = radiusInterpolation(t);
						var v = valueInterpolation(t);

						if (serie.horizontal) {
							fontSize = 30;

							var w = maxWidthInterpolation(t);

							obj.attr("x", w / 2)
								.attr("y", r * serie.progressWidth / 2 + _labelRadius / 2);
						} else {
							if (objWidth > r * 0.9) {
								fontSize--;
							} else if (objWidth < r * 0.9) {
								fontSize++;
							}
						}

						if (serie.showProgressValue == "%") {
							obj.text((v * 100 / serie.maxValue).formatFloat(2)+"%");
						} else {
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									v = serie.valueFormat(v);
								} else {
									v = formatValue(v, serie.valueFormat);
								}
							}
							obj.text(v);
						}

						return fontSize;
					};
				});
		}

		if (serie.showValueLabels) {
			var label = progressContainer.selectAll("text.label-value");
			
			if (label.empty()) {
				label = progressContainer.append("text")
					.attr("class", "label-value")
					.style("font-size", "11px")
					.attr("dy", "0.35em");
			}

			label.transition().duration(_duration)
				.each("end", function(d) {
					var obj = d3.select(this);
					if (serie.showValueLabels == "%"){
						obj.text((serie.value * 100 / serie.maxValue).formatFloat(2)+"%");
					} else {
						var value = serie.value;
						if (serie.valueFormat) {
							if (isFunction(serie.valueFormat)) {
								value = serie.valueFormat(value);
							} else {
								value = formatValue(value, serie.valueFormat);
							}
						}
						obj.text(value);
					}
				})
				.attrTween("transform", function(d) {
					var obj = d3.select(this);
					if (isUndefinedOrNull(this._current)) {
						this._current = { angle: 0, radius: radius, value: 0, maxWidth: maxWidth };
					}
					var angle = Math.PI * 2 * Math.min(1, serie.value / serie.maxValue);
					var angleInterpolation = d3.interpolate(this._current.angle, angle);
					var radiusInterpolation = d3.interpolate(this._current.radius, radius);
					var valueInterpolation = d3.interpolate(this._current.value, serie.value);
					var maxWidthInterpolation = d3.interpolate(this._current.maxWidth, maxWidth);
					this._current.angle = angle;
					this._current.radius = radius;
					this._current.value = serie.value;
					this._current.maxWidth = maxWidth;

					return function(t) {
						var x, y, percent, a;
						var r = radiusInterpolation(t);

						if (serie.horizontal) {
							var v = valueInterpolation(t);
							var w = maxWidthInterpolation(t);
							percent = v * 100 / serie.maxValue;

							x = w * percent / 100;
							y = (-r * serie.progressWidth / 2) - (_labelRadius / 2);
						} else {
							a = angleInterpolation(t);

							percent = a * 100 / (Math.PI * 2);

							x = Math.sin(a) * (r + _labelRadius / 2);
							y = Math.cos(a) * (r + _labelRadius / 2) * -1;							
						}

						var anchor;
						if (serie.horizontal) {
							anchor = "middle";
						} else {
							if (Math.round(Math.sin(a)) < 0) {
								anchor = "end";	
							} else if (Math.round(Math.sin(a)) === 0) {
								anchor = "middle";
							} else {
								anchor = "start";
							}
						}
						obj.attr("text-anchor", anchor);
						
						if (serie.showValueLabels == "%") {
							obj.text(percent.formatFloat(2)+"%");
						} else {
							var value = serie.maxValue * percent / 100;
							if (serie.valueFormat) {
								if (isFunction(serie.valueFormat)) {
									value = serie.valueFormat(value);
								} else {
									value = formatValue(value, serie.valueFormat);
								}
							}
							obj.text(value);
						}

						return "translate(" + x + ", " + y + ")";
					};
				});
		}
	}

	function hasAxis(serie) {
		return serie.type == "bar" || serie.type == "line" || serie.type == "area";
	}

	function initSeries() {
		var barSeries = false;
		var lineAreaSeries = false;
		var circularSeries = false; // pie, donut, gauge
		var stackedFloatingSeries = false;

		chart.series.forEach(function(serie, i) {
			serie = chart.series[i] = jQuery.extend(true, {}, defaultSerieOptions, defaultSeriesOptions[serie.type], serie); 
			if (isUndefinedOrNull(serie._serieContainer)) {
				serie._serieContainer = svgContainer.append("g")
					.attr("id","serie-"+i)
					.attr("class", "serie-container");			

				if (hasAxis(serie)) {
					serie._serieContainer.append("g").attr("class","grid");
				}

				serie._serieContainer._dataContainer = serie._serieContainer.append("g")
					.attr("clip-path", "url(#clip-path-" + chart.id + ")")
					.append("g")
					.attr("class","data-container");

				if (hasAxis(serie)) {
					serie._serieContainer.append("g").attr("class","axis y-axis");
				}
			}

			barSeries = barSeries || serie.type == "bar";
			lineAreaSeries = lineAreaSeries || serie.type == "line" || serie.type == "area";
			circularSeries = circularSeries || serie.type == "pie" || serie.type == "donut" || serie.type == "gauge" || serie.type == "progress";
			stackedFloatingSeries = stackedFloatingSeries || (serie.stacked && serie.floating);

			if (serie.colors && typeof serie.colors == "string") {
				serie.colors = serie.colors.split("_");
			}
			if (serie.fields && typeof serie.fields == "string") {
				serie.fields = serie.fields.split("_");
			}
			if (serie.labels && typeof serie.labels == "string") {
				serie.labels = serie.labels.split("_");
			}

			if (hasAxis(serie)) {
				var dataMaxValue;
				var dataMinValue;
				
				if (chart.dataSource.data.length === 0) {
					dataMaxValue = 0;
					dataMinValue = 0;
				} else {
					var fieldsMaxValue = {};
					if (serie.stacked) {
						dataMaxValue = d3.max(chart.dataSource.data, function(d) {
							var stackedValue = 0;
							serie.fields.forEach(function(field) {
								stackedValue += d[field];
								d["_stackedValue_"+field] = stackedValue;

								if (serie.floating) {
									fieldsMaxValue[field] = Math.max(fieldsMaxValue[field] || Number.MIN_SAFE_INTEGER, d[field]);
								}
							});
							d._stackedValue = stackedValue;
							return stackedValue;
						});
						dataMinValue = 0;

						if (serie.floating) {
							serie._fieldsMaxValue = fieldsMaxValue;
						}
					} else {
						dataMaxValue = d3.max(chart.dataSource.data, function(d) {
							return d3.max(serie.fields, function(f) { return d[f]; });
						});
						dataMinValue = Math.min(0, d3.min(chart.dataSource.data, function(d) {
							return d3.min(serie.fields, function(f) { return d[f]; });
						}));
					}
				}

				serie._minValue = serie.yAxis.range.min || dataMinValue;
				serie._maxValue = serie.yAxis.range.max || dataMaxValue;

				if (!isUndefinedOrNull(serie.yAxis.linkedSerie)) {
					var linkedSerie = chart.series[serie.yAxis.linkedSerie];
					linkedSerie._minValue = linkedSerie.yAxis.range.min || Math.min(dataMinValue, linkedSerie._minValue);
					linkedSerie._maxValue = linkedSerie.yAxis.range.max || Math.max(dataMaxValue, linkedSerie._maxValue);
				}

				if (chart.invertedAxes) {
					if (i > 0) {
						serie.linkedSerie = 0;
					}
				}
			}

			if (serie.type == "pie" || serie.type == "donut") {
				serie.pieData = [];
				if (serie.pivoted && chart.dataSource.data.length > 0) {
					serie.pivotedFields.forEach(function(field, i) {
						serie.pieData.push({
							label: serie.pivotedLabels[i],
							value: chart.dataSource.data[0][field] || 0
						});
					});

					serie.fieldValue = "value";
					serie.fieldLabel = "label";
				} else {
					serie.pieData = chart.dataSource.data;
				}

				serie._sumData = d3.sum(serie.pieData, function(d) {
					if (isUndefinedOrNull(d[serie.fieldValue])) {
						d[serie.fieldValue] = 0;
					}
					return d[serie.fieldValue];
				});

				if (serie.sort !== false) {
					serie.pieData.sort(function(a, b) {
						var field = serie.sort == "label" ? serie.fieldLabel : serie.fieldValue;
						return b[field] - a[field];
					});
				}
			}

			if (serie.type == "gauge") {
				chart.showTooltip = false;
				chart.legend.visible = false;

				if (serie.rangeType == "%" && isUndefinedOrNull(serie.maxValue)) {
					throw new Error("Gauge with percentual ranges must have a maxValue set.");
				}

				serie._gaugeData = [];
				var rangeValue;
				for (var idx = 1; idx < serie.ranges.length; idx++) {
					rangeValue = serie.ranges[idx] - serie.ranges[idx - 1];
					if (serie.rangeType == "%") {
						rangeValue = rangeValue * serie.maxValue / 100;
					}
					serie._gaugeData.push({
						rangeSize: rangeValue,
						rangeMinValue: serie.ranges[idx - 1],
						rangeMaxValue: serie.ranges[idx]
					}); 
				}

				serie._minValue = serie.ranges[0];
				serie._maxValue = serie.ranges[serie.ranges.length - 1];
				if (serie.rangeType == "%") {
					serie._minValue = serie._minValue * serie.maxValue / 100;
					serie._maxValue = serie._maxValue * serie.maxValue / 100;
				}

				if (serie.maxValue && serie._maxValue !== serie.maxValue) {
					serie._gaugeData.push({
						rangeSize: serie.maxValue - serie._maxValue,
						rangeMinValue: serie._maxValue,
						rangeMaxValue: serie.maxValue
					});
					serie._maxValue = serie.maxValue;
				}
			} 

			if (serie.type == "progress") {
				chart.showTooltip = false;
				chart.legend.visible = false;
			}
		});

		if (barSeries) {
			chart.xAxis.type = "bands";
		} else {
			chart.xAxis.type = "points";
		}

		if (circularSeries) {
			if (chart.series.length > 1) {
				throw new Error("Pie, donut, gauge and progress series cannot be associated with other series");
			}

			chart.xAxis.visible = false;
		}

		if (stackedFloatingSeries) {
			// stacked floating series cannot show legend
			chart.legend.visible = false;
		}
	}

	function draw() {
		resizeChart();

		drawTitle();
		drawSubTitle();

		preRenderAxes();		// prerender axes to calc axes dimensions
		
		calcChartDimensions();	// calc chart dimensions considering axes dimensions
		drawAxes();				// draw axes	

		checkTitleOverflow();

		drawSeries();
		drawLegend();

		if (chart.events.onFinishDrawing) {
			chart.events.onFinishDrawing();
		}
	}

	function needData() {
		var result = false;
		chart.series.forEach(function(serie, i) {
			result = result || hasAxis(serie) || serie.type == "pie" || serie.type == "donut";
		});

		return result;
	}

	function drawSeries() {
		var drawFunction = {
			bar: drawBarSerie,
			line: drawLineSerie,
			area: drawAreaSerie,
			pie: drawPieDonutSerie,
			donut: drawPieDonutSerie, 
			gauge: drawGaugeSerie,
			progress: drawProgressSerie
		};

		svgContainer.select("text.empty-text").remove();

		if (needData() && chart.dataSource.data.length === 0) {
			svgContainer.append("text")
				.attr("class","empty-text")
				.attr("x", chart._chartWidth / 2)
				.attr("y", chart._chartHeight / 2)
				.attr("text-anchor", "middle")
				.text("No data.");

			chart.series.forEach(function(serie) {
				serie._serieContainer._dataContainer.selectAll("*").remove();
			});
		} else {
			chart.series.forEach(function(serie, i) {
				drawFunction[serie.type](serie);
			});
		}
	}

	function calcChartDimensions() {
		chart._chartHeight = svgContainerHeight - chart.padding.top - chart.padding.bottom - titlePadding - axisDimensions.top - axisDimensions.bottom;
		chart._chartWidth = svgContainerWidth - chart.padding.left - chart.padding.right - axisDimensions.left - axisDimensions.right;
		chart._chartTop = chart.padding.top + titlePadding + axisDimensions.top;
		chart._chartLeft = chart.padding.left + axisDimensions.left;

		chart.series.forEach(function(serie, i) {
			serie._serieContainer.attr("transform", "translate("+chart._chartLeft + ", " + chart._chartTop +")");
		});

		if (chart._xAxisContainer && chart._chartWidth < chart._xAxisContainer.select("path.domain").node().getBBox().width) {
			chart.clippingPath.transition().duration(_duration)
				.attr("width", chart._chartWidth + 2)
				.attr("height", svgContainerHeight)
				.attr("transform", "translate(-1, -" + titlePadding + ")");
		} else {
			chart.clippingPath.transition().duration(_duration)
				.attr("width", chart._chartWidth + 20)
				.attr("height", svgContainerHeight)
				.attr("transform", "translate(-10, -" + titlePadding + ")");
		}

		svgContainer.select("rect.panning-controller")
			.attr("width", chart._chartWidth)
			.attr("height", chart._chartHeight)
			.attr("transform", "translate(" + chart._chartLeft + ", " + chart._chartTop + ")");
	}

	function preRenderAxes() {
		var renderOrder = chart.invertedAxes ? ["x", "y"] : ["y", "x"];

		renderOrder.forEach(function(axis) {
			calcChartDimensions();	// calc chart dimensions without axes dimensions

			if (axis == "y") {
				chart.series.forEach(function(serie, i) {
					if ((serie.yAxis && serie.yAxis.visible !== false && hasAxis(serie)) || chart.invertedAxes) {
						initSerieYScale(serie);
						drawSerieYAxis(serie);
						if (serie.yAxis.visible !== false) {
							if (!chart.invertedAxes) {
								axisDimensions[serie.yAxis.position] = Math.max(axisDimensions[serie.yAxis.position], serie._yAxisContainer.node().getBBox().width);
							} else {
								axisDimensions[serie.yAxis.position] = Math.max(axisDimensions[serie.yAxis.position], serie._yAxisContainer.node().getBBox().width - chart._startPosition);
							}
						}
					}
				});
			} else {
				if ((chart.xAxis && chart.xAxis.visible !== false) || chart.invertedAxes) {
					initChartXScale();
					drawChartXAxis();
					if (chart.xAxis.visible !== false) {
						if (!chart.invertedAxes) {
							axisDimensions[chart.xAxis.position] = Math.max(axisDimensions[chart.xAxis.position], chart._startPosition + chart._xAxisContainer.node().getBBox().height - chart._chartHeight);
						} else {
							axisDimensions[chart.xAxis.position] = Math.max(axisDimensions[chart.xAxis.position], chart._xAxisContainer.node().getBBox().height);
						}
					}
				}
			}
		});
	}

	function drawAxes() {
		chart.series.forEach(function(serie, i) {
			if (serie.yAxis && (serie.yAxis.visible !== false || !isUndefinedOrNull(serie.yAxis.linkedSerie)) && hasAxis(serie)) {
				initSerieYScale(serie);
				drawSerieYAxis(serie);	
			}
		});

		if (chart.xAxis && chart.xAxis.visible !== false) {
			initChartXScale();
			drawChartXAxis();
		}
	}

	function initChartXScale() {
		var bandWidth;

		if (!chart.invertedAxes) {
			if (chart.xAxis.type == "bands") {
				if (chart.xAxis.bandDimensions.type == "auto") {
					chart._xScale = d3.scale.ordinal()
						.rangeRoundBands([0, chart._chartWidth], chart.xAxis.bandDimensions.gap, 0);
				} else {
					if (chart.dataSource.data.length !== 0) {
						bandWidth = Math.max(chart._chartWidth / chart.dataSource.data.length, chart.xAxis.bandDimensions.minWidth);
						
						chart._xScale = d3.scale.ordinal()
							.rangeRoundBands([0, bandWidth * chart.dataSource.data.length], chart.xAxis.bandDimensions.gap, 0);
					} else {
						bandWidth = chart._chartWidth;

						chart._xScale = d3.scale.ordinal()
							.rangeRoundBands([0, bandWidth], chart.xAxis.bandDimensions.gap, 0);
					}
	
				}
			} else if (chart.xAxis.type == "points") {
				if (chart.xAxis.bandDimensions.type == "auto") {
					chart._xScale = d3.scale.ordinal()
						.rangeRoundPoints([0, chart._chartWidth], chart.xAxis.bandDimensions.gap);		
				} else {
					if (chart.dataSource.data.length > 1) {
						bandWidth = Math.max(chart._chartWidth / (chart.dataSource.data.length - 1), chart.xAxis.bandDimensions.minWidth);
		
						chart._xScale = d3.scale.ordinal()
							.rangeRoundPoints([0, bandWidth * (chart.dataSource.data.length - 1)], chart.xAxis.bandDimensions.gap);
					} else {
						bandWidth = chart._chartWidth;						
		
						chart._xScale = d3.scale.ordinal()
							.rangeRoundPoints([0, bandWidth], chart.xAxis.bandDimensions.gap);
					}
				}
			}
	
			chart._xScale.domain(chart.dataSource.data.map(function(d) { return d[chart.xAxis.field]; }));
	
			chart._bandLeft = function(data) {
				return chart._xScale(data[chart.xAxis.field]);
			};
			chart._bandWidth = function() {
				return chart._xScale.rangeBand();
			};
			chart._pointLeft = function(data) {
				return chart._bandLeft(data) + chart._bandWidth() / 2;
			};

			// init chart start position to draw elements (bars, lines)
			chart._startPosition = chart.series[0]._yScale ? chart.series[0]._yScale(chart.series[0]._startValue) : chart._chartHeight;
		} else {
			var _minValue, _maxValue;

			_minValue = d3.min(chart.series, function(serie) { return serie._minValue; });
			_maxValue = d3.max(chart.series, function(serie) { return serie._maxValue; });

			chart._xScale = d3.scale.linear()
				.range([0, chart._chartWidth])
				.domain([_minValue, _maxValue]);

			chart._startValue = _minValue < 0 ? 0 : _minValue;

			// init chart start position to draw elements (bars, lines)
			chart._startPosition = chart._xScale(chart._startValue);
		}
	}

	function xAxisOverflow() {
		var textSpace;
		var textWidths = [];

		if (chart.xAxis.type == "bands") {
			textSpace = chart._bandWidth();
		} else {
			textSpace = 0;
		}

		chart._xAxisContainer.selectAll("text").each(function() { textWidths.push(this.getBBox().width); });

		return d3.max(textWidths) > textSpace;
	}

	function wrapTicks(text) {
		var maxWidth;

		if (!isUndefinedOrNull(chart.xAxis.ticks.maxWidth) && !isUndefinedOrNull(chart.xAxis.ticks.maxLines)) {
			maxWidth = chart.xAxis.ticks.maxWidth;

			text.each(function() {
				var text = d3.select(this),
					words = text.text().split(/\s+/).reverse(),
					word, splitWord,
					line = [],
					lineNumber = 0,
					lineHeight = 1.1,  //ems
					y = text.attr("y"),
					dy = parseFloat(text.attr("dy")),
					tspan = text.text(null).append("tspan")
						.attr("x", 0)
						.attr("y", y)
						.attr("dy", dy + "em")
						.attr("dx",text.attr("dx"));

				word = words.pop();
				while (!isUndefinedOrNull(word)) {
					line.push(word);
					tspan.text(line.join(" "));
					
					if (tspan.node().getComputedTextLength() > maxWidth) {
						if (lineNumber === chart.xAxis.ticks.maxLines - 1) {
							break;
						}

						line.pop();

						if (line.length > 0) {
							tspan.text(line.join(" "));
							line = [word];
							tspan = text.append("tspan")
								.attr("x", 0)
								.attr("y", y)
								.attr("dy", ++lineNumber * lineHeight + dy + "em")
								.attr("dx",text.attr("dx"))
								.text(word);
						} else {
							if (words.length > 0) {
								splitWord = "";
								while (tspan.node().getComputedTextLength() > maxWidth) {
									splitWord = word.substr(-1) + splitWord;
									word = word.slice(0, -1);
									tspan.text(word);
								}
								word = splitWord;
								line = [word];

								tspan = text.append("tspan")
									.attr("x", 0)
									.attr("y", y)
									.attr("dy", ++lineNumber * lineHeight + dy + "em")
									.attr("dx",text.attr("dx"))
									.text(word);
							}
						}
					}

					word = words.pop();
				}

				// adjust last line
				word = tspan.text();
				while (word.length > 0 && tspan.node().getComputedTextLength() > maxWidth) {
					word = word.slice(0, -1);
					tspan.text(word+"...");
				}
			});
		}
	}

	function drawXAxis(xAxis, xAxisYPosition) {
		var axisSelection;
		chart._xAxisContainer = svgContainer.select("g.x-axis");
		if (chart._xAxisContainer.empty()) {
			chart._xAxisContainer = svgContainer.append("g")
				.style('opacity', 0)
				.attr("class", "axis x-axis")
				.attr("transform", "translate(0, " + xAxisYPosition + ")");

			axisSelection = chart._xAxisContainer.append("g")
				.attr("class","ticks-container")
				.attr("clip-path", "url(#clip-path-" + chart.id + ")")
				.append("g")
				.attr("class","ticks");
		} else {
			var xAxisSelection;
			if (chart._xAxisInitialized) {
				xAxisSelection = chart._xAxisContainer.transition().duration(_duration);
			} else {
				xAxisSelection = chart._xAxisContainer;				
				chart._xAxisInitialized = true;
			}
			axisSelection = xAxisSelection
				.style('opacity', 1)
				.attr("transform", "translate(" + chart._chartLeft + ", " + xAxisYPosition + ")");
		}

		chart._xAxisContainer.select("g.ticks").call(xAxis);

		chart._xAxisContainer.selectAll("path, line")
			.style("fill","none")
			.style("stroke","#999")
			.style("shape-rendering", "crispEdges");

		if (chart.invertedAxes) {
			chart._xAxisContainer.selectAll("path.domain")
				.style("display", "none");
		}
		
		chart._xAxisContainer.selectAll("text")
			.style("font-size", "10px")
			.call(wrapTicks);

		if (!chart.invertedAxes) {
			chart._xAxisContainer.selectAll("text")
				.on("mouseover", showTooltip)
				.on("mousemove", showTooltip)
				.on("mouseout", hideTooltip);
		}
	}

	function addFloatingChartAxis(serie) {
		var _scale = d3.scale.ordinal()
			.rangeRoundBands([0, chart._chartWidth])
			.domain(serie.labels);

		var xAxis = d3.svg.axis()
			.scale(_scale)
			.orient(chart.xAxis.position);

		var xAxisYPosition;
		if (chart.xAxis.position == "top") {
			xAxisYPosition = chart._chartTop;
		} else {
			xAxisYPosition = chart._chartHeight + chart._chartTop;
		}

		drawXAxis(xAxis, xAxisYPosition);

		chart._xAxisContainer.selectAll("path.domain")
			.style("display", "block");

		chart._xAxisContainer.selectAll("text")
			.style("text-anchor", "start");

		chart._xAxisContainer.selectAll("g.tick").each(function(d, i) {
			var tick = d3.select(this);
			tick.classed("tick-"+i, true);
		});
	}

	function drawChartXAxis() {
		if (chart.xAxis && chart.xAxis.visible !== false) {
			if (chart.series[0].stacked && chart.series[0].floating && chart.invertedAxes) {
				addFloatingChartAxis(chart.series[0]);
			} else {
				var xAxis = d3.svg.axis()
					.scale(chart._xScale)
					.orient(chart.invertedAxes ? "bottom" : chart.xAxis.position);

				var xAxisYPosition;
				if (!chart.invertedAxes) {
					if (chart.xAxis.position == "top") {
						xAxisYPosition = 0;
					} else {
						xAxisYPosition = chart._startPosition + chart._chartTop;
					}

					if (chart.xAxis.format) {
						if (isFunction(serie.xAxis.format)) {
							xAxis.tickFormat(chart.xAxis.format);
						} else {
							xAxis.tickFormat(function(value) { return formatValue(value, chart.xAxis.format); });
						}
					}
				} else {
					xAxisYPosition = chart._chartHeight + chart._chartTop;

					if (chart.series[0].yAxis.format) {
						if (isFunction(chart.series[0].yAxis.format)) {
							xAxis.tickFormat(chart.series[0].yAxis.format);
						} else {
							xAxis.tickFormat(function(value) { return formatValue(value, chart.series[0].yAxis.format); });
						}
					}
				}

				xAxis.ticks(Math.max(chart._chartWidth/50, 2));
				if (chart.invertedAxes && chart.series[0].yAxis.formatName == "number" && (chart._xScale.domain()[1] - chart._xScale.domain()[0] < xAxis.ticks()[0])) {
					xAxis.tickValues(d3.range(chart._xScale.domain()[0], chart._xScale.domain()[1]+1, 1));
				}

				drawXAxis(xAxis, xAxisYPosition);

				// rotation must be between [0..90] degrees
				chart.xAxis.ticks.rotation = chart.xAxis.ticks.rotation > 90 ? chart.xAxis.ticks.rotation % 90 : chart.xAxis.ticks.rotation;
				if (chart.xAxis.type == "points" || chart.xAxis.ticks.rotation || chart.invertedAxes || xAxisOverflow()) {
					chart._xAxisContainer.selectAll('text')
						.attr("transform", "rotate(-" + (chart.xAxis.ticks.rotation || 60) + ")")
						.attr("dx", "-.8em")
						.attr("dy", ".15em")
						.style("text-anchor", "end")
						.selectAll("tspan")
						.attr("y", 0)
						.attr("x", -2);
				} else {
					chart._xAxisContainer.selectAll('text')
						.attr("transform", null)
						.attr("dx", null)
						.attr("dy", ".71em")
						.style("text-anchor", "middle");
				}

				if (chart.invertedAxes) {
					chart._xAxisContainer.select('text')
						.style("display", "none");
				}

				if (chart.xAxis.label) {
					var deltaY = { top: -1, bottom: 1};

					var label = chart._xAxisContainer.select("text.label");
					if (label.empty()) {
						label = chart._xAxisContainer.append("text")
							.style('opacity', 0)
							.attr("class","label")
							.attr("text-anchor","middle")
							.attr("x",chart._chartWidth/2)
							.attr("y",(chart._xAxisContainer.select("g.ticks").node().getBBox().height + 20) * deltaY[chart.xAxis.position])
							.text(chart.xAxis.label);
					} else {
						var xLabelSelection;
						if (chart._xLabelInitialized) {
							xLabelSelection = label.transition().duration(_duration);
						} else {
							xLabelSelection = label;				
							chart._xLabelInitialized = true;
						}
						xLabelSelection
							.style('opacity', 1)
							.attr("x",chart._serieWidth/2)
							.attr("y",(chart._xAxisContainer.select("g.ticks").node().getBBox().height + 20) * deltaY[chart.xAxis.position]);
					}
				}

				if (chart.invertedAxes) {
					if (chart.series[0].yAxis.showGridLines) {
						var xGrid = d3.svg.axis()
							.scale(chart._xScale)
							.orient(chart.xAxis.position)
							.tickSize(-chart._chartHeight, 0, 0)
							.tickFormat("")
							.ticks(Math.max(chart._chartWidth/50, 2));

						if (isUndefinedOrNull(chart._xGridContainer)) {
							chart._xGridContainer = svgContainer.select("g.x-grid")
								.style('opacity', 0)
								.attr("class", "x-grid")
								.attr("transform", "translate(" + chart._chartLeft + ", " + xAxisYPosition + ")")
								.call(xGrid);

							chart._xGridContainer.selectAll(".tick")
								.style("stroke", "#ccc")
								.style("stroke-opacity", 0.7)
								.style("shape-rendering", "crispEdges");

							chart._xGridContainer.selectAll("path")
								.style("display", "none");
						} else {
							var gridSelection;
							if (chart._xGridInitialized) {
								gridSelection = chart._xGridContainer.transition().duration(_duration);
							} else {
								gridSelection = chart._xGridContainer;				
								chart._xGridInitialized = true;
							}
							gridSelection
								.style("opacity", 1)
								.attr("transform", "translate(" + chart._chartLeft + ", " + xAxisYPosition + ")")
								.call(xGrid);

							chart._xGridContainer.selectAll(".tick")
								.style("stroke", "#ccc")
								.style("stroke-opacity", 0.7)
								.style("shape-rendering", "crispEdges");

							chart._xGridContainer.selectAll("path")
								.style("display", "none");
						}
					}
				}
			}

			setupPanning(true);
		}
	}

	function mouseDownPanning() {
		if (d3.event.type == "mousedown") {
			chartPanning = { x: d3.event.clientX };
		} else {
			chartPanning = { x: d3.event.touches[0].clientX };
		}
	}

	function mouseMovePanning() {
		var _x, deltaX;
		if (chartPanning && !legendDragging) {
			if (d3.event.type == "mousemove") {
				_x = d3.event.clientX;
			} else {
				_x = d3.event.touches[0].clientX;
			}
			deltaX = _x - chartPanning.x;
			if (Math.abs(deltaX) > 0) {
				panChart(deltaX);
				chartPanning = { x: _x, panned: true };
			}
		}
	}

	function mouseUpPanning() {
		if (chartPanning.panned) {
			d3.event.stopPropagation();
		}
		chartPanning = false;
	}

	function setupPanning(enabled) {
		if (chart._xAxisContainer && chart.scrollable) {
			enabled = enabled && chart._chartWidth < chart._xAxisContainer.select("path.domain").node().getBBox().width;

			if (enabled) {
				svgContainer.on("mousedown.panning", mouseDownPanning);
				svgContainer.on("mousemove.panning", mouseMovePanning);
				svgContainer.on("mouseup.panning", mouseUpPanning);
				svgContainer.on("touchstart.panning", mouseDownPanning);
				svgContainer.on("touchmove.panning", mouseMovePanning);
				svgContainer.on("touchend.panning", mouseUpPanning);

				drawScrollBar();
			} else {
				svgContainer.on("mousedown.panning", null);
				svgContainer.on("mousemove.panning", null);
				svgContainer.on("mouseup.panning", null);
				svgContainer.on("touchstart.panning", null);
				svgContainer.on("touchmove.panning", null);
				svgContainer.on("touchend.panning", null);

				hideScrollBar();
			}
		}
	}

	function hideScrollBar() {
		svgContainer.select("g.scroll-bar").style("display", "none");
	}

	function drawScrollBar() {
		var scrollbarContainer = svgContainer.select("g.scroll-bar");

		if (scrollbarContainer.empty()) {
			scrollbarContainer = svgContainer.append("g")
				.attr("class", "scroll-bar")
				.attr("transform", "translate(0, " + (svgContainerHeight - 10) + ")");

			scrollbarContainer.append("rect")
				.attr("fill", "#eee")
				.attr("class", "scroll-bar-track")
				.attr("height", 3);

			scrollbarContainer.append("rect")
				.attr("fill", "#aaa")
				.attr("class", "scroll-bar-thumb")
				.attr("height", 3);
		}

		scrollbarContainer.style("display", "block").transition().duration(_duration)
			.attr("transform", "translate(0, " + (svgContainerHeight - 10) + ")");

		scrollbarContainer.select("rect.scroll-bar-track")
			.attr("width", chart._chartWidth)
			.attr("transform", "translate(" + chart._chartLeft + ", 0)");

		scrollbarContainer.select("rect.scroll-bar-thumb")
			.attr("width", chart._chartWidth * chart._chartWidth / chart._xAxisContainer.select("path.domain").node().getBBox().width)
			.attr("transform", "translate(" + chart._chartLeft + ", 0)");
	}

	function initSerieYScale(serie) {
		if (!chart.invertedAxes) {
			if (isUndefinedOrNull(serie.yAxis.linkedSerie)) {
				serie._yScale = d3.scale.linear()
					.range([chart._chartHeight, 15]) // 15px is the space needed for value labels
					.domain([serie._minValue, serie._maxValue]);

				serie._startValue = serie._minValue < 0 ? 0 : serie._minValue;
			} else {
				var linkedSerie = chart.series[serie.yAxis.linkedSerie];

				serie.yAxis.visible = false;
				serie.yAxis.format = linkedSerie.yAxis.format;

				serie._minValue = linkedSerie._minValue;
				serie._maxValue = linkedSerie._maxValue;
				serie._startValue = linkedSerie._startValue;
				serie._yScale = linkedSerie._yScale;
			}
		} else {
			var bandHeight;

			if (chart.xAxis.type == "bands") {
				if (chart.xAxis.bandDimensions.type == "auto") {
					serie._yScale = d3.scale.ordinal()
						.rangeRoundBands([0, chart._chartHeight], chart.xAxis.bandDimensions.gap, 0);
				} else {
					if (chart.dataSource.data.length !== 0) {
						bandHeight = chart._chartHeight / chart.dataSource.data.length;
		
						serie._yScale = d3.scale.ordinal()
							.rangeRoundBands([0, bandHeight * chart.dataSource.data.length], chart.xAxis.bandDimensions.gap, 0);
					} else {
						bandHeight = chart._chartHeight;
		
						serie._yScale = d3.scale.ordinal()
							.rangeRoundBands([0, bandHeight], chart.xAxis.bandDimensions.gap, 0);
					}
				}
			} else if (chart.xAxis.type == "points") {
				if (chart.xAxis.bandDimensions.type == "auto") {
					serie._yScale = d3.scale.ordinal()
						.rangeRoundPoints([0, chart._chartHeight], chart.xAxis.bandDimensions.gap);		
				} else {
					if (chart.dataSource.data.length > 1) {
						bandHeight = chart._chartHeight / (chart.dataSource.data.length - 1);
		
						serie._yScale = d3.scale.ordinal()
							.rangeRoundPoints([0, bandHeight * (chart.dataSource.data.length - 1)], chart.xAxis.bandDimensions.gap);
					} else {
						bandHeight = chart._chartHeight;
		
						serie._yScale = d3.scale.ordinal()
							.rangeRoundPoints([0, bandHeight], chart.xAxis.bandDimensions.gap);
					}
				}
			}
	
			serie._yScale.domain(chart.dataSource.data.map(function(d) { return d[chart.xAxis.field]; }));
	
			serie._bandTop = function(data) {
				return serie._yScale(data[chart.xAxis.field]);
			};
			serie._bandHeight = function() {
				return serie._yScale.rangeBand();
			};
			serie._pointTop = function(d) {
				return serie._bandTop(d) + serie._bandHeight() / 2;
			};
		}
	}

	function drawYAxis(serie, yAxis, yAxisPosition) {
		var axisSelection;
		if (isUndefinedOrNull(serie._yAxisContainer)) {
			serie._yAxisContainer = serie._serieContainer.select("g.y-axis")
				.style('opacity', 0)
				.attr("class", "axis y-axis")
				.attr("transform", "translate(" + yAxisPosition + ",0)");

			axisSelection = serie._yAxisContainer.append("g")
				.attr("class","ticks")
				.call(yAxis);

		} else {
			var yAxisSelection;
			if (serie._yAxisInitialized) {
				yAxisSelection = serie._yAxisContainer.transition().duration(_duration);
			} else {
				yAxisSelection = serie._yAxisContainer;				
				serie._yAxisInitialized = true;
			}
			axisSelection = yAxisSelection
				.attr("transform", "translate(" + yAxisPosition + ",0)")
				.style('opacity', 1)
				.select("g.ticks")
				.call(yAxis);
		}

		serie._yAxisContainer.selectAll("path, line")
			.style("fill","none")
			.style("stroke","#999")
			.style("shape-rendering", "crispEdges");

		if (!chart.invertedAxes) {
			serie._yAxisContainer.selectAll("path.domain")
				.style("display", "none");
		}

		serie._yAxisContainer.selectAll("text")
			.style("font-size", "10px");

		if (chart.invertedAxes) {
			serie._yAxisContainer.selectAll("text")
				.on("mouseover", showTooltip)
				.on("mousemove", showTooltip)
				.on("mouseout", hideTooltip);
		}
	}

	function addFloatingSerieAxis(serie) {
		var _scale = d3.scale.ordinal()
			.rangeRoundBands([chart._chartHeight, 0])
			.domain(serie.labels);

		var yAxis = d3.svg.axis()
			.scale(_scale)
			.orient(serie.yAxis.position);

		var yAxisPosition;
		if (serie.yAxis.position == "left") {
			yAxisPosition = 0;
		} else {
			yAxisPosition = chart._chartWidth;
		}

		drawYAxis(serie, yAxis, yAxisPosition);

		serie._yAxisContainer.selectAll("path.domain")
			.style("display", "block");

		serie._yAxisContainer.selectAll("g.tick").each(function(d, i) {
			var tick = d3.select(this);
			tick.classed("tick-"+i, true);
		});
	}

	function drawSerieYAxis(serie) {
		if (serie.yAxis && serie.yAxis.visible !== false) {
			if (serie.stacked && serie.floating && !chart.invertedAxes) {
				addFloatingSerieAxis(serie);
			} else {
				var yAxis = d3.svg.axis()
					.scale(serie._yScale)
					.orient(chart.invertedAxes ? "left" : serie.yAxis.position);

				var yAxisPosition;
				if (!chart.invertedAxes) {
					yAxis.ticks(Math.max(chart._chartHeight/40, 2));

					if (serie.yAxis.formatName == "number" && (serie._maxValue - serie._minValue < yAxis.ticks()[0])) {
						yAxis.tickValues(d3.range(serie._minValue, serie._maxValue+1, 1));
					}

					if (serie.yAxis.position == "left") {
						yAxisPosition = 0;
					} else {
						yAxisPosition = chart._chartWidth;
					}

					if (serie.yAxis.format) {
						if (isFunction(serie.yAxis.format)) {
							yAxis.tickFormat(serie.yAxis.format);
						} else {
							yAxis.tickFormat(function(value) { return formatValue(value, serie.yAxis.format); });
						}
					}
				} else {
					yAxisPosition = chart._xScale(chart._startValue);
					if (chart.xAxis.format) {
						if (isFunction(chart.xAxis.format)) {
							yAxis.tickFormat(chart.xAxis.format);
						} else {
							yAxis.tickFormat(function(value) { return formatValue(value, chart.xAxis.format); });
						}
					}
				}

				drawYAxis(serie, yAxis, yAxisPosition);

				if (!chart.invertedAxes) {
					if (serie.yAxis.showGridLines && serie == chart.series[0]) {
						var yGrid = d3.svg.axis()
							.scale(serie._yScale)
							.orient(serie.yAxis.position)
							.tickSize(-chart._chartWidth, 0, 0)
							.tickFormat("")
							.ticks(Math.max(chart._chartHeight/40, 2));

						if (isUndefinedOrNull(serie._gridContainer)) {
							serie._gridContainer = serie._serieContainer.select("g.grid")
								.style('opacity', 0)
								.attr("class", "grid")
								.attr("transform", "translate(" + yAxisPosition + ",0)")
								.call(yGrid);

							serie._gridContainer.selectAll(".tick")
								.style("stroke", "#ccc")
								.style("stroke-opacity", 0.7)
								.style("shape-rendering", "crispEdges");

							serie._gridContainer.selectAll("path")
								.style("display", "none");
						} else {
							var gridSelection;
							if (serie._gridInitialized) {
								gridSelection = serie._gridContainer.transition().duration(_duration);
							} else {
								gridSelection = serie._gridContainer;				
								serie._gridInitialized = true;
							}
							gridSelection
								.style("opacity", 1)
								.attr("transform", "translate(" + yAxisPosition + ",0)")
								.call(yGrid);

							serie._gridContainer.selectAll(".tick")
								.style("stroke", "#ccc")
								.style("stroke-opacity", 0.7)
								.style("shape-rendering", "crispEdges");

							serie._gridContainer.selectAll("path")
								.style("display", "none");
						}
					}

					if (serie.yAxis.label) {
						var deltaX = { left: -1, right: 1};
						var rotation = { left: -90, right: 90};

						var label = serie._yAxisContainer.select("text.label");
						if (label.empty()) {
							label = serie._yAxisContainer.append("text")
								.style('opacity', 0)
								.attr("class","label")
								.attr("text-anchor","middle")
								.attr("transform","rotate(" + rotation[serie.yAxis.position] + ")")
								.attr("x",chart._chartHeight/2 * deltaX[serie.yAxis.position])
								.attr("y",(serie._yAxisContainer.select("g.ticks").node().getBBox().width + 10) * -1)
								.style("font-weight", 700)
								.style("font-size", "12px")
								.text(serie.yAxis.label);

								if (serie.fields.length === 1) {
									label.attr("fill", serie.colors[0 % serie.colors.length]);
								}
						} else {
							var yLabelSelection;
							if (serie._yLabelInitialized) {
								yLabelSelection = label.transition().duration(_duration);
							} else {
								yLabelSelection = label;				
								serie._yLabelInitialized = true;
							}
							yLabelSelection
								.style('opacity', 1)
								.attr("x",chart._chartHeight/2 * deltaX[serie.yAxis.position])
								.attr("y",(serie._yAxisContainer.select("g.ticks").node().getBBox().width + 10) * -1);
						}
					}
				}
			}
		}
	}

	var legendDragging = false;
	var LegendTop = 0;
	var legendLeft = 0;

	function drawLegend() {
		var legendContainer;
		var handlerRadius = 12;
		var labelHeight = 10;
		var labelSymbolWidth = 20;
		var labelMargin = 5;
		var boxMargin = 10;
		var boxWidth = 0;
		var boxHeight = boxMargin;

		var drawLegendItem = function(serie, label, serieIdx, labelIdx) {
			var legendLabel = legendContainer.select("g.legend-label-"+serieIdx+"-"+labelIdx);
			if (legendLabel.empty()) {
				legendLabel = legendContainer.append("g")
					.attr("class","legend-label legend-label-"+serieIdx+"-"+labelIdx)
					.attr("transform", "translate(" + (boxMargin + handlerRadius) + ", " + boxHeight + ")");

				if (serie.type == "bar") {
					legendLabel.append("rect")
						.attr("width", labelSymbolWidth)
						.attr("height", labelHeight)
						.attr("fill", serie.colors[labelIdx % serie.colors.length]);
				}

				if (serie.type == "line" || serie.type == "area") {
					var	symbol = d3.svg.symbol().type(serie.marks.type).size(Math.pow(labelHeight-2,2));

					if (serie.type == "area") {
						legendLabel.append("rect")
							.attr("width", labelSymbolWidth)
							.attr("height", labelHeight / 2)
							.attr("y", labelHeight / 2)
							.attr("opacity", serie.areaOpacity)
							.attr("fill", serie.colors[labelIdx % serie.colors.length]);							
					}

					legendLabel.append("path")
						.attr("transform", "translate(" + labelSymbolWidth/2 + ", " + labelHeight/2 + ")")
						.attr("d", symbol)
						.attr("fill", serie.colors[labelIdx % serie.colors.length]);

					legendLabel.append("line")
						.attr("stroke-width", Math.min(serie.lineWidth, 3))
						.attr("stroke", serie.colors[labelIdx % serie.colors.length])
						.attr("x1", 0).attr("y1", labelHeight/2)
						.attr("x2", labelSymbolWidth).attr("y2", labelHeight/2);
				}

				if (serie.type == "pie" || serie.type == "donut" || serie.type == "gauge") {
					legendLabel.append("path")
						.attr("fill", serie.colors[labelIdx % serie.colors.length])
						.attr("stroke-width", 0)
						.attr("transform", "translate(" + labelSymbolWidth / 2 + ", " + labelHeight + ")")
						.attr("d", function() {
							var arc = d3.svg.arc()
								.innerRadius(labelHeight / 2.5)
								.outerRadius(labelHeight)
								.startAngle(-60 * (Math.PI/180))
								.endAngle(60 * (Math.PI/180));
							return arc();
						});
				}

				if (serie.type == "pie" || serie.type == "donut") {
					legendLabel.attr("field-idx",labelIdx)
						.on("mouseup", function(d) {
							if (!legendDragging) {
								var idx = d3.select(this).attr("field-idx");
								dataClick(serie.pieData[idx]);
							}
						});
				}

				legendLabel.append("text")
					.attr("dx", labelSymbolWidth + labelMargin)
					.attr("y", labelHeight / 2)
					.attr("dy", "0.35em")
					.style("font-size", "10px")
					.text(label);
			}

			boxWidth = Math.max(boxWidth, Math.round(legendLabel.node().getBBox().width) + boxMargin * 2);
			boxHeight += Math.round(legendLabel.node().getBBox().height) + labelMargin;
		};

		function moveLegend(animate) {
			legendLeft = Math.max(chart.legend.margin, legendLeft);
			legendLeft = Math.min(svgContainerWidth - chart.legend.margin - legendContainer.node().getBBox().width, legendLeft);

			LegendTop = Math.max(chart.legend.margin, LegendTop);
			LegendTop = Math.min(svgContainerHeight - chart.legend.margin - legendContainer.node().getBBox().height, LegendTop);

			if (animate) {
				legendContainer.transition().duration(_duration)
					.attr("transform", "translate(" + legendLeft + ", " + LegendTop + ")"); 
			} else {				
				legendContainer.attr("transform", "translate(" + legendLeft + ", " + LegendTop + ")"); 
			}
		}

		if (chart.legend) {
			legendContainer = svgContainer.select("g.legend");
			if (legendContainer.empty()) {
				legendContainer = svgContainer.append("g")
					.attr("class","legend")
					.style("cursor","move");	

				var drag = d3.behavior.drag()
					.origin(Object)
					.on('dragstart', function() { 
						legendContainer.style("opacity", 0.5); 
					})
					.on('drag', function() { 
						legendLeft += d3.event.dx;
						LegendTop += d3.event.dy;

						if (!legendDragging) {
							legendDragging = true;
							setupPanning(false);
						}

						moveLegend(false);
					})
					.on('dragend', function() { 
						legendContainer.style("opacity", 1); 
						legendDragging = false;
						setupPanning(true);
					});

				legendContainer.call(drag);				
			} else {
				legendContainer.html("");
			}

			var legendBox = legendContainer.select("rect.legend-box");
			if (legendBox.empty()) {
				legendBox = legendContainer.append("rect")
					.attr("class","legend-box")
					.attr("y", handlerRadius)
					.attr("x", handlerRadius)
					.attr("fill", "white")
					.attr("fill-opacity", 0.7)
					.attr("stroke", "#ccc")
					.attr("stroke-width", 1)
					.attr("shape-rendering", "crispEdges");

				boxHeight += 12;
			}

			var legendHandler = legendContainer.select("g.legent-handler");
			if (legendHandler.empty()) {
				legendHandler = legendContainer.append("g")
					.attr("class", "legend-handler")
					.on("mouseup", function() {
						if (!legendDragging) {
							chart.legend.closed = !chart.legend.closed;
							drawLegend();
							moveLegend(true); // prevent legend to be braw over the limits of svg
						}
					});

				legendHandler.append("circle")
					.attr("cx", handlerRadius)
					.attr("cy", handlerRadius)
					.attr("r", handlerRadius)
					.attr("fill", "#ccc");

				legendHandler.append("line")
					.attr("stroke-width", 2)
					.attr("stroke", "white")
					.attr("x1", handlerRadius/2)
					.attr("y1", handlerRadius)
					.attr("x2", handlerRadius + handlerRadius/2)
					.attr("y2", handlerRadius);

				if (chart.legend.closed) {
					legendHandler.append("line")
						.attr("stroke-width", 2)
						.attr("stroke", "white")
						.attr("x1", handlerRadius)
						.attr("y1", handlerRadius/2)
						.attr("x2", handlerRadius)
						.attr("y2", handlerRadius + handlerRadius/2);

					var closedText = legendContainer.append("text")
						.attr("font-size", "12px")
						.attr("y", boxHeight)
						.attr("dy", "0.7em")
						.attr("x", boxMargin + handlerRadius)
						.text("LEGENDA"); 

					boxWidth = Math.max(boxWidth, Math.round(closedText.node().getBBox().width) + boxMargin * 2);
					boxHeight += Math.round(closedText.node().getBBox().height - 3);
				}
			}

			if (!chart.legend.closed) {
				chart.series.forEach(function(serie, serieIdx) {
					if (hasAxis(serie)) {
						var _labels = serie.labels.slice(0);
						if (serie.stacked && !chart.invertedAxes) {
							_labels.reverse();
						}

						_labels.forEach(function(label, labelIdx) {
							if (serie.stacked && !chart.invertedAxes) {
								labelIdx = _labels.length - 1 - labelIdx;
							}
							drawLegendItem(serie, label, serieIdx, labelIdx);
						});					
					} else {
						if (serie.type == "pie" || serie.type == "donut") {
							serie.pieData.forEach(function(d, labelIdx) {
								label = d[serie.fieldLabel];
								drawLegendItem(serie, label, serieIdx, labelIdx);
							});
						} else {
							chart.dataSource.data.forEach(function(d, labelIdx) {
								label = d[serie.fieldLabel];
								drawLegendItem(serie, label, serieIdx, labelIdx);
							});
						}
					}
				});
			}

			boxHeight += boxMargin - labelMargin;

			legendBox.attr("height", boxHeight - handlerRadius)
				.attr("width", boxWidth);

			if (!chart._legendInitialized) {
				var boxPosition = chart.legend.position.split(" ");
				if (boxPosition[0] == "top") {
					LegendTop = chart._chartTop + chart.legend.margin;
				} else if (boxPosition[0] == "bottom") {
					LegendTop = chart._chartTop + chart._chartHeight - boxHeight - chart.legend.margin;
				} else if (boxPosition[0] == "center") {
					LegendTop = (chart._chartTop + chart._chartHeight - boxHeight) / 2;
				}
				if (boxPosition[1] == "left") {
					legendLeft = chart._chartLeft + chart.legend.margin;
				} else if (boxPosition[1] == "right") {
					legendLeft = chart._chartLeft + chart._chartWidth - boxWidth - chart.legend.margin;
				} else if (boxPosition[1] == "center") {
					legendLeft = (chart._chartLeft + chart._chartWidth - boxWidth) / 2;
				}
				legendContainer.attr("transform", "translate(" + legendLeft +  ", " + LegendTop + ")");

				chart._legendInitialized = true;
			} else {
				moveLegend(true); // prevent legend to be braw over the limits of svg
			}

			legendContainer.style("display", chart.legend.visible !== false ? "block" : "none");
		}
	}

	var panningX = 0;
	function panChart(deltaX) {
		if (chart.xAxis && chart.xAxis.visible !== false) {
			panningX += deltaX;
			panningX = Math.max(Math.min(panningX, 0), chart._chartWidth - chart._xAxisContainer.node().getBBox().width);
			
			chart.series.forEach(function(serie) {
				serie._serieContainer._dataContainer.attr("transform", "translate(" + panningX + ", 0)");
			});

			chartContainer.select("g.x-axis").select("g.ticks").attr("transform", "translate(" + panningX + ", 0)");

			var scrollbarContainer = svgContainer.select("g.scroll-bar");
			var scrollbarThumb = scrollbarContainer.select("rect.scroll-bar-thumb");
			var scrollbarDelta = panningX * chart._chartWidth / chart._xAxisContainer.node().getBBox().width;

			scrollbarThumb.attr("transform", "translate(" + (chart._chartLeft - scrollbarDelta) + ", 0)");
		}
	}

	function resetPanning() {
		if (chart.xAxis && chart.xAxis.visible !== false) {
			panningX = 0;

			chart.series.forEach(function(serie) {
				serie._serieContainer._dataContainer.transition().duration(_duration).attr("transform", "translate(" + panningX + ", 0)");
			});

			chartContainer.select("g.x-axis").select("g.ticks").transition().duration(_duration).attr("transform", "translate(" + panningX + ", 0)");
		}
	}

	function dataClick(data) {
		if (!(chartPanning && chartPanning.panned) && chart.events.onDataClick) {
			chart.events.onDataClick(d3.event, data);
		}
	}

	var lastTooltipData;
	var tooltipOffset = 5;

	function showTooltip(d, dataIdx) {
		if (chart.showTooltip && !legendDragging && !isMobile()) {
			if (typeof d == "string") {
				d = chart.dataSource.data[dataIdx];
			}

			if (d !== lastTooltipData) {
				lastTooltipData = d;

				var html = "";
				var serieData = "";

				chart.series.forEach(function(serie, idx) {
					if (hasAxis(serie)) {
						if (idx === 0) {
							html += "<span class=\"x-label\">" + d[chart.xAxis.field] + "</span><br />";
						}

						serie.fields.forEach(function(field, i) {
							if (!isUndefinedOrNull(d[field])) {
								var value = d[field];
								if (serie.yAxis.format) {
									if (isFunction(serie.yAxis.format)) {
										value = serie.yAxis.format(value);
									} else {
										value = formatValue(value, serie.yAxis.format);
									}
								}
								if (serie.yAxis.label) {
									value = value + " " + serie.yAxis.label;
								}
								if (serie.stacked && !chart.invertedAxes) {
									serieData = serie.labels[i] + ": " + value + "<br />" + serieData;
								} else {
									serieData = serieData + serie.labels[i] + ": " + value + "<br />";
								}
							}
						});
					} else if (serie.type == "pie" || serie.type == "donut") {
						serieData += d.data[serie.fieldLabel] + ": ";
						if (serie.valueFormat) {
							if (isFunction(serie.valueFormat)) {
								serieData += serie.valueFormat(d.data[serie.fieldValue]);
							} else {
								serieData += formatValue(d.data[serie.fieldValue], serie.valueFormat);
							}
						} else {
							serieData += d.data[serie.fieldValue];
						}
					}
				});
				html += serieData;
				chart._tooltipContainer.html(html);
			}

			var left = d3.event.clientX + tooltipOffset - $(chartContainer.node()).offset().left;
			if (left + chart._tooltipContainer.node().clientWidth > chartContainer.node().clientWidth) {
				left = d3.event.clientX - tooltipOffset - $(chartContainer.node()).offset().left - chart._tooltipContainer.node().clientWidth;
			}
			var top = d3.event.clientY + tooltipOffset - $(chartContainer.node()).offset().top;
			if (top + chart._tooltipContainer.node().clientHeight > chartContainer.node().clientHeight) {
				top = d3.event.clientY - tooltipOffset - $(chartContainer.node()).offset().top - chart._tooltipContainer.node().clientHeight;
			}
			chart._tooltipContainer
				.style("opacity", 1)
				.style("left", left + "px")
				.style("top", top + "px");
		}
	}

	function hideTooltip(d) {
		chart._tooltipContainer.style("opacity", 0)
			.style("left", -1000 + "px")
			.style("top", -1000 + "px");
	}

	function initTooltip() {
		chart._tooltipContainer = d3.select(chartContainer.node().parentNode).select(".zh-chart-tooltip");
	}

	function _chart(element) {
		initChart(element);

		initSeries();
		draw();
	}

	_chart.redraw = buildDebounceMethod(function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			if (chart.scrollable) {
				resetPanning();
			}
			draw();
		}
	}, 100);

	_chart.savePNG = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			var uri = this.getBase64();

			var image = new Image();
			image.onload = function() {
				var canvas = document.createElement('canvas');
				canvas.width = image.width;
				canvas.height = image.height;
				var context = canvas.getContext('2d');
				context.drawImage(image, 0, 0);

				canvas.toBlobHD(function(blob) {
					saveAs(blob, "chart_"+(new Date().getTime())+".png");
				}, "image/png");
			};
			image.onerror = function() {
				console.error(
					'There was an error loading the data URI as an image on the following SVG\n',
					window.atob(uri.slice(26)), '\n',
					"Open the following link to see browser's diagnosis\n",
					uri);
			};
			image.src = uri;
		}
	};

	_chart.getBase64 = function() {
		function reEncode(data) {
			data = encodeURIComponent(data);
			data = data.replace(/%([0-9A-F]{2})/g, function(match, p1) {
				var c = String.fromCharCode('0x'+p1);
				return c === '%' ? '%25' : c;
			});
			return decodeURIComponent(data);
		}

		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			var html = this.getSVG();
			var uri = 'data:image/svg+xml;base64,' + window.btoa(reEncode(html));

			return uri;
		}
	};

	_chart.saveSVG = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			try {
				var isFileSaverSupported = !!new Blob();
			} catch (e) {
				alert("blob not supported");
			}

			var html = this.getSVG();

			var blob = new Blob([html], {type: "image/svg+xml"});
			saveAs(blob, "chart_"+(new Date().getTime())+".svg");
		}
	};

	_chart.getSVG = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			var html = chartContainer.select("svg")
				.attr("title", chart.title)
				.attr("version", 1.1)
				.attr("xmlns", "http://www.w3.org/2000/svg")
				.node().parentNode.innerHTML;

			return html;
		}
	};

	_chart.toggleTooltip = function(show) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			chart.showTooltip = !isUndefinedOrNull(show) ? show : !chart.showTooltip;
		}
	};

	_chart.toggleLegend = function(show) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			chart.legend.visible = !isUndefinedOrNull(show) ? show : !chart.legend.visible;
			svgContainer.select("g.legend").style("display", chart.legend.visible ? "block" : "none");
		}
	};

	_chart.toggleValueLabels = function(show) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			chart.showValueLabels = !isUndefinedOrNull(show) ? show : !chart.showValueLabels;
			svgContainer.selectAll(".label-value").style("display", chart.showValueLabels ? "block" : "none");
		}
	};

	function showDrillUpButton() {
		if (chart.showDrillUpButton) {
			var button = svgContainer.select(".drill-up-button");
			if (button.empty()) {
				button = svgContainer.append("g")
					.attr("class","drill-up-button")
					.attr("transform", "translate(" + chart.padding.left + ", " + chart.padding.left + ")")
					.style("cursor", "pointer")
					.on("click", function() {
						_chart.drillUp();
					});

				var rect = button.append("rect")
					.attr("fill","#333")
					.attr("x", 0)
					.attr("y", 0);

				var text = button.append("text")
					.attr("x", 10)
					.attr("y", 10)
					.attr("dy", "1em")
					.attr("font-size", 12)
					.attr("fill", "white")
					.text("Drill Up");

				rect.attr("width", text.node().getBBox().width + 20)
					.attr("height", text.node().getBBox().height + 20);
			}

			button.style("display", "block");
		}
	}

	function hideDrillUpButton() {
		svgContainer.select(".drill-up-button").style("display", "none");
	}

	_chart.drillDown = function(options) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			var currentChart = jQuery.extend(true, {}, chart);
			drillPath.push(currentChart);
			showDrillUpButton();

			chart.dataSource.data = [];
			chart = jQuery.extend(true, chart, options);

			initSeries();
			draw();
		}
	};

	_chart.drilledDown = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			return drillPath.length > 0;
		}
	};

	_chart.drillUp = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			if (drillPath.length > 0) {
				chart = drillPath.splice(-1,1)[0];
				if (drillPath.length === 0) {
					hideDrillUpButton();
				}

				initSeries();
				draw();
			}
		}
	};

	_chart.updateChart = function(newOptions) {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			chart.dataSource.data = [];
			options = newOptions;

			initChart();			

			initSeries();
			draw();
		}
	};

	_chart.value = function(value) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(value)) {
				options.series[0].value = value;
			}
			return _chart;
		} else {
			var serie = chart.series[0]; // gauge and progress are the first and only serie
			if (isUndefinedOrNull(value)) {
				return serie.value;
			} else {
				if (serie.type == "gauge") {
					serie.value = Math.max(value, serie._minValue);
				} else if (serie.type == "progress") {
					serie.value = Math.min(Math.max(value, 0), serie.maxValue);
				}
				return _chart;
			}
		}
	};

	_chart.maxValue = function(value) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(value)) {
				options.series[0].maxValue = value;
			}
			return _chart;
		} else {
			var serie = chart.series[0]; // gauge and progress are the first and only serie
			if (isUndefinedOrNull(value)) {
				return serie.maxValue;
			} else {
				serie.maxValue = value;
				return _chart;
			}
		}
	};

	_chart.gaugeRanges = function(ranges) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(ranges)) {
				options.series[0].ranges = ranges;
			}
			return _chart;
		} else {
			var serie = chart.series[0]; // gauge is the first and only serie
			if (isUndefinedOrNull(ranges)) {
				return serie.ranges;
			} else {
				serie.ranges = ranges;
				return _chart;
			}
		}
	};

	_chart.dataSourceData = function(data) {
		if (isUndefinedOrNull(chart)) {
			if (!isUndefinedOrNull(data)) {
				options.dataSource.data = data;
			}
			return _chart;
		} else {
			if (isUndefinedOrNull(data)) {
				return chart.dataSource.data;
			} else {
				chart.dataSource.data = data;
				return _chart;
			}
		}
	};

	_chart.updateSeries = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			initSeries();
			draw();
		}
	};


	_chart.clear = function() {
		if (isUndefinedOrNull(chart)) {
			return _chart;
		} else {
			this.dataSourceData([]).updateSeries();
		}
	};

	return _chart;
}

$(window).on("resize.chart", function() { 
	$(".zh-chart-container:visible").each(function(idx, element) {
		if (element._chart) {
			element._chart.redraw();
		}
	});
});
